///////////////////////////////////////////////////////////
//														 //
// AquaFlyer.cpp										 //
// Aqua Flyer II game main file							 //
// Developer:	Robert Rak								 //
// Class:		CIS487									 //
// Date:		12/21/2005								 //
//														 //
///////////////////////////////////////////////////////////

// INCLUDES ///////////////////////////////////////////////

#define INITGUID

#define WIN32_LEAN_AND_MEAN  

#include <windows.h>		// include important windows stuff
#include <windowsx.h> 
#include <mmsystem.h>
#include <iostream.h>		// include important C/C++ stuff
#include <conio.h>
#include <stdlib.h>
#include <malloc.h>
#include <memory.h>
#include <string.h>
#include <stdarg.h>
#include <stdio.h> 
#include <math.h>
#include <io.h>
#include <fcntl.h>

#include <ddraw.h>			// directX includes
#include <dsound.h>
#include <dmksctrl.h>
#include <dmusici.h>
#include <dmusicc.h>
#include <dmusicf.h>
#include <dinput.h>
#include "T3DLIB1.h"		// game library includes
#include "T3DLIB2.h"
#include "T3DLIB3.h"

// DEFINES ////////////////////////////////////////////////

// defines for windows 
#define WINDOW_CLASS_NAME "WINCLASS"	// class name

#define WINDOW_WIDTH    640				// size of window viewport
#define WINDOW_HEIGHT   480

#define BITMAP_ID            0x4D42		// universal id for a bitmap
#define MAX_COLORS_PALETTE   256		// number of colors in palette

// size of universe
#define UNIVERSE_MIN_X   (0)			// min and max dimensions			
#define UNIVERSE_MAX_X   (640)			// of the univers
#define UNIVERSE_MIN_Y   (0)
#define UNIVERSE_MAX_Y   (3392)

// weapons defines
#define MISSLE_MAX			64			// max number of missles fired
#define MISSLE_SPEED_SLOW	6			// pixels/sec of fired missle - slow
#define MISSLE_SPEED_MEDIUM	8			// medium speed
#define MISSLE_SPEED_FAST	10			// fast speed
#define MISSLE_STATE_OFF	0			// missle is not fired
#define MISSLE_STATE_ON		1			// missle is in flight
#define MISSLE_RANGE_FLYER	30			// range of player missle
#define MISSLE_RANGE_ENEMY	30			// range of enemy missle
#define MISSLE_ANIM_FLYER	0
#define MISSLE_ANIM_ENEMY	1

// explosion defines 
#define EXPLOSION_MAX		8			// max explosions at one time
#define EXPLOSION_STATE_OFF	0			// explosion is inactive
#define EXPLOSION_STATE_ON	1			// explosion is active
#define EXPLOSION_WIDTH		48
#define EXPLOSION_HEIGHT	48

// player ship states defines
#define FLYER_STATE_DEAD		0		// ship is destroyed
#define FLYER_STATE_DYING		1		// ship is low on fuel
#define FLYER_STATE_INVINCIBLE	2		// ship is reactivated
#define FLYER_STATE_ALIVE		3		// ship is in good state
#define FLYER_SPEED_MAX			(-5)		// max ship speed
#define FLYER_SPEED_MIN			(-1)		// min ship speed
#define FLYER_WEAPON_OFF		0
#define FLYER_WEAPON_ON			1
#define FLYER_DIR_LEFT			0
#define FLYER_DIR_NORMAL		1
#define FLYER_DIR_RIGHT			2

// patrol boat defines
#define BOAT_MAX			10
#define BOAT_STATE_DEAD		0
#define BOAT_STATE_ALIVE	1
#define BOAT_WEAPON_OFF		0
#define BOAT_WEAPON_ON		1
#define BOAT_SPEED_MAX		3
#define BOAT_SPEED_MIN		2
#define BOAT_DIR_LEFT		0
#define BOAT_DIR_RIGHT		1
#define BOAT_POINTS			300

// cruiser defines
#define CRUISER_MAX			10
#define CRUISER_STATE_DEAD	0
#define CRUISER_STATE_ALIVE	1
#define CRUISER_WEAPON_OFF	0
#define CRUISER_WEAPON_ON	1
#define CRUISER_SPEED_MAX	4
#define CRUISER_SPEED_MIN	2
#define CRUISER_DIR_LEFT	0
#define CRUISER_DIR_RIGHT	1
#define CRUISER_POINTS		400

// destroyer defines
#define DESTROYER_MAX			10
#define DESTROYER_STATE_DEAD	0
#define DESTROYER_STATE_ALIVE	1
#define DESTROYER_WEAPON_OFF	0
#define DESTROYER_WEAPON_ON		1
#define DESTROYER_SPEED_MAX		3
#define DESTROYER_SPEED_MIN		2
#define DESTROYER_DIR_LEFT		0
#define DESTROYER_DIR_RIGHT		1
#define DESTROYER_POINTS		500

// helicopter defines
#define HELICOPTER_MAX			10
#define HELICOPTER_STATE_DEAD	0
#define HELICOPTER_STATE_ALIVE	1
#define HELICOPTER_WEAPON_OFF	0
#define HELICOPTER_WEAPON_ON	1
#define HELICOPTER_SPEED_MAX	4
#define HELICOPTER_SPEED_MIN	3
#define HELICOPTER_DIR_LEFT		0
#define HELICOPTER_DIR_RIGHT	1
#define HELICOPTER_POINTS		600

// subsonic plane defines
#define PLANE_MAX			10
#define PLANE_STATE_DEAD	0
#define PLANE_STATE_ALIVE	1
#define PLANE_WEAPON_OFF	0
#define PLANE_WEAPON_ON		1
#define PLANE_SPEED_MAX		5
#define PLANE_SPEED_MIN		4
#define PLANE_DIR_LEFT		0
#define PLANE_DIR_RIGHT		1
#define PLANE_POINTS		700

// supersonic jet defines
#define JET_MAX			10
#define JET_STATE_DEAD	0
#define JET_STATE_ALIVE	1
#define JET_WEAPON_OFF	0
#define JET_WEAPON_ON	1
#define JET_SPEED_MAX	6
#define JET_SPEED_MIN	5
#define JET_DIR_LEFT	0
#define JET_DIR_RIGHT	1
#define JET_POINTS		800

// fuel station defines
#define FUEL_MAX			10
#define FUEL_STATE_DEAD		0
#define FUEL_STATE_ALIVE	1
#define FUEL_POINTS			200

// bridge defines
#define BRIDGE_STATE_DEAD	0
#define BRIDGE_STATE_ALIVE	1
#define BRIDGE_POINTS		2000

// texture defines
#define NUM_TILES_WORLD			32
#define NUM_TILES_FLYER			3
#define NUM_TILES_BOAT			2
#define NUM_TILES_CRUISER		2
#define NUM_TILES_DESTROYER		2
#define NUM_TILES_HELICOPTER	2
#define NUM_TILES_PLANE			2
#define NUM_TILES_JET			2
#define NUM_TILES_FUEL			1
#define NUM_TILES_MISSLE		16
#define NUM_TILES_EXPLOSION		14
#define NUM_TILES_BRIDGE		1


// tile size defines
#define WORLD_TILE_WIDTH		64
#define WORLD_TILE_HEIGHT		64
#define FLYER_TILE_WIDTH		40
#define FLYER_TILE_HEIGHT		48
#define BOAT_TILE_WIDTH			80
#define BOAT_TILE_HEIGHT		24
#define CRUISER_TILE_WIDTH		112
#define CRUISER_TILE_HEIGHT		24
#define DESTROYER_TILE_WIDTH	112
#define DESTROYER_TILE_HEIGHT	40
#define HELICOPTER_TILE_WIDTH	64
#define HELICOPTER_TILE_HEIGHT	24
#define PLANE_TILE_WIDTH		48
#define PLANE_TILE_HEIGHT		16
#define JET_TILE_WIDTH			48
#define JET_TILE_HEIGHT			16
#define MISSLE_TILE_WIDTH		8
#define MISSLE_TILE_HEIGHT		8
#define EXPLOSION_TILE_WIDTH	48
#define EXPLOSION_TILE_HEIGHT	48
#define FUEL_TILE_WIDTH			48
#define FUEL_TILE_HEIGHT		96
#define BRIDGE_TILE_WIDTH		128
#define BRIDGE_TILE_HEIGHT		64

// color defines
#define COLOR_RIVER_BANK	197
#define COLOR_RIVER_BANK_2	230
#define COLOR_TRANSPARENT	0

// sound defines
#define MAX_MISSLE_SOUNDS		8			// max missle sounds
#define MAX_EXPLOSION_SOUNDS	8			// max explosion sounds

// data field in BOBs
#define INDEX_STATE		0
#define INDEX_SPEED		1
#define INDEX_WEAPON	2
#define INDEX_DIR		3
#define INDEX_WORLD_X	4
#define INDEX_WORLD_Y	5

// enemies fields defines
#define ENEMY_TYPE		0
#define POS_X			2
#define POS_Y			3
#define WEAPON_STATE	1

// enemy type defines
#define TYPE_BOAT		0
#define TYPE_CRUISER	1
#define TYPE_DESTROYER	2
#define TYPE_HELICOPTER	3
#define TYPE_PLANE		4
#define TYPE_JET		5
#define TYPE_FUEL		6

// game state defines
#define MAIN_SCREEN			0
#define NEW_GAME_SCREEN		1
#define INSTRUCTION_SCREEN	2
#define GAME_FINISH_SCREEN	4
#define GAME_PLAY			5
#define GAME_RESET			6
#define EXIT_GAME			7
#define EXIT_FADE			8
#define FLYER_EXPLODE		9
#define NEW_GAME_INIT		10
#define GAME_OVER			11
#define NEXT_LEVEL			12

// game difficulty defines
#define EASY		0
#define MODERATE	1
#define DIFFICULT	2

#define MAX_LEVELS	10

#define FIRE_PAUSE_TIME	15

// font defines
#define FONT_NUM_CHARS         96  // entire character set
#define FONT_WIDTH             12
#define FONT_HEIGHT            12
#define FONT_WIDTH_NEXT_NUM    8
#define FONT_WIDTH_NEXT_LOWER  7
#define FONT_WIDTH_NEXT_UPPER  8

// PROTOTYPES /////////////////////////////////////////////

// game console
int  Game_Init(void *parms=NULL);
int  Game_Main(void *parms=NULL);
int  Game_Shutdown(void *parms=NULL);
void Game_Reset(void);

void Handle_Main_Screen(void);
void Handle_New_Game_Screen(void);
void Handle_Instruction_Screen(void);
void Handle_Game_Finish_Screen(void);
void Handle_Game_Over_Screen(void);
void Handle_Exit_Game(void);
void Handle_Game_Play(void);
void Handle_Flyer_Explode(void);
void Handle_New_Game_Init(void);
void Handle_Next_Level(void);

// game logic functions
void Init_Boats(void);
void Start_Boats(void);
void Move_Boats(void);
void Draw_Boats(void);
void Reset_Boats(void);
void Delete_Boats(void);

void Init_Cruisers(void);
void Start_Cruisers(void);
void Move_Cruisers(void);
void Draw_Cruisers(void);
void Reset_Cruisers(void);
void Delete_Cruisers(void);

void Init_Destroyers(void);
void Start_Destroyers(void);
void Move_Destroyers(void);
void Draw_Destroyers(void);
void Reset_Destroyers(void);
void Delete_Destroyers(void);

void Init_Helicopters(void);
void Start_Helicopters(void);
void Move_Helicopters(void);
void Draw_Helicopters(void);
void Reset_Helicopters(void);
void Delete_Helicopters(void);

void Init_Planes(void);
void Start_Planes(void);
void Move_Planes(void);
void Draw_Planes(void);
void Reset_Planes(void);
void Delete_Planes(void);

void Init_Jets(void);
void Start_Jets(void);
void Move_Jets(void);
void Draw_Jets(void);
void Reset_Jets(void);
void Delete_Jets(void);

void Init_Screens(void);
void Delete_Screens(void);

void Init_World(void);
void Draw_World(void);
void Delete_World(void);

void Init_Flyer(void);
void Start_Flyer(void);
void Move_Flyer(void);
void Draw_Flyer(void);
void Delete_Flyer(void);

void Init_Explosions(void);
void Start_Explosions(int x, int y, int width, int height, int xv, int yv);
void Move_Explosions(void);
void Draw_Explosions(void);
void Reset_Explosions(void);
void Delete_Explosions(void);

void Init_Missles(void);
void Fire_Missles(int x, int y, int xv, int yv, int source);
void Move_Missles(void);
void Draw_Missles(void);
void Reset_Missles(void);
void Delete_Missles(void);

void Init_Fuel(void);
void Start_Fuel(void);
void Move_Fuel(void);
void Draw_Fuel(void);
void Reset_Fuel(void);
void Delete_Fuel(void);

void Init_Bridge(void);
void Start_Bridge(void);
void Move_Bridge(void);
void Draw_Bridge(void);
void Reset_Bridge(void);
void Delete_Bridge(void);

void Init_Info(void);
void Draw_Info(void);
void Delete_Info(void);

int Init_Sounds(void);
void Start_Missle_Enemy_Sound(int x, int y, int width, int height);
void Start_Missle_Flyer_Sound(int x, int y, int width, int height);
void Start_Explosion_Sound(void);

inline void Copy_Palette(LPPALETTEENTRY dest, LPPALETTEENTRY source)
{ memcpy(dest, source, 256*sizeof(PALETTEENTRY)); }

// MACROS ////////////////////////////////////////////////

#define GRID(c,w) ((c)/64+(w)/64)
#define RAND_RANGE(x,y) ( (x) + (rand()%((y)-(x)+1)))

// used to compute the min and max of two expresions
#define MIN(a, b)  (((a) < (b)) ? (a) : (b)) 
#define MAX(a, b)  (((a) > (b)) ? (b) : (a)) 

// GLOBALS ////////////////////////////////////////////////

HWND main_window_handle = NULL; // save the window handle
HINSTANCE main_instance = NULL; // save the instance
char buffer[80];                // used to print text

// bobs for all game graphics
BOB world;
BOB flyer;
BOB boats[BOAT_MAX];
BOB cruisers[CRUISER_MAX];
BOB destroyers[DESTROYER_MAX];
BOB helicopters[HELICOPTER_MAX];
BOB planes[PLANE_MAX];
BOB jets[JET_MAX];
BOB explosions[EXPLOSION_MAX];
BOB missles[MISSLE_MAX];
BOB fuel[FUEL_MAX];
BOB bridge;
BOB info;

// bitmap images for menus
BITMAP_IMAGE main_screen;
BITMAP_IMAGE new_game_screen;
BITMAP_IMAGE instruction_screen;
BITMAP_IMAGE game_finish_screen;
BITMAP_IMAGE game_over_screen;

// color palette holders
PALETTEENTRY menu_pal[256];
PALETTEENTRY world_pal[256];

// game specific variables
int curr_game_state = MAIN_SCREEN;
int curr_game_difficulty = EASY;
int curr_game_level = 0;

// current world position coordinates
int world_x = 0;
int world_y = 3391;

// aqua flyer specific coordinates
int flyer_dx = 0;
int flyer_dy = 0;
int flyer_init_x = 288;
int flyer_init_y = 416;
int flyer_fuel = 1000;
int flyer_ships;
int flyer_score;
int flyer_explode = 0;
int flyer_fire_counter = FIRE_PAUSE_TIME;
int flyer_engine_on = 0;

// bridge coordinates
int bridge_init_world_x = 288;
int bridge_init_world_y = 32;

// sound ids
int engine_id = -1;
int fueling_id = -1;
int fuel_low_id = -1;

int explosion_ids[MAX_EXPLOSION_SOUNDS] = {-1,-1,-1,-1,-1,-1,-1,-1};
int missle_enemy_ids[MAX_MISSLE_SOUNDS] = {-1,-1,-1,-1,-1,-1,-1,-1};
int missle_flyer_ids[MAX_MISSLE_SOUNDS] = {-1,-1,-1,-1,-1,-1,-1,-1};

// enemy specific variables
int num_enemies[10] = {20, 20, 25, 25, 30, 30, 30, 30, 30, 30};

int enemies[10][30][4] = 
{
	{
		{0, 0, 200, 200},
		{1, 0, 220, 370},
		{6, 0, 240, 540},
		{2, 0, 220, 710},
		{1, 0, 220, 880},
		{6, 0, 270, 1050},
		{0, 0, 220, 1220},
		{1, 0, 240, 1390},
		{6, 0, 240, 1560},
		{1, 0, 220, 1730},
		{2, 0, 240, 1900},
		{6, 0, 240, 2070},
		{0, 0, 220, 2240},
		{3, 0, 230, 2410},
		{6, 0, 250, 2580},
		{3, 0, 220, 2750},
		{0, 0, 220, 2900},
		{6, 0, 230, 3090},
		{0, 0, 220, 3280},
		{0, 0, 230, 3430},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0}
	},
	{
		{1, 0, 200, 200},
		{3, 0, 220, 370},
		{6, 0, 240, 540},
		{2, 0, 220, 710},
		{1, 1, 220, 880},
		{6, 0, 270, 1050},
		{1, 0, 220, 1220},
		{0, 0, 240, 1390},
		{6, 0, 240, 1560},
		{2, 1, 200, 1760},
		{3, 0, 240, 1900},
		{6, 0, 240, 2070},
		{0, 0, 220, 2240},
		{2, 0, 220, 2410},
		{6, 0, 250, 2580},
		{1, 0, 230, 2750},
		{2, 0, 190, 2940},
		{6, 0, 230, 3090},
		{1, 0, 220, 3260},
		{0, 0, 230, 3430},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0}
	},
	{
		{3, 1, 220, 180},
		{1, 1, 220, 320},
		{0, 0, 220, 460},
		{6, 0, 220, 600},
		{0, 0, 220, 740},
		{1, 0, 210, 880},
		{4, 0, 200, 1020},
		{6, 0, 250, 1160},
		{3, 1, 210, 1300},
		{1, 0, 220, 1440},
		{2, 1, 200, 1580},
		{0, 0, 200, 1720},
		{6, 0, 230, 1860},
		{1, 1, 210, 2000},
		{0, 0, 210, 2140},
		{3, 1, 210, 2280},
		{6, 0, 230, 2400},
		{2, 0, 220, 2540},
		{3, 0, 210, 2680},
		{1, 0, 220, 2800},
		{6, 0, 230, 2940},
		{3, 0, 210, 3080},
		{2, 0, 220, 3220},
		{0, 0, 210, 3360},
		{4, 0, 220, 3500},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0}
	},
	{
		{1, 1, 220, 180},
		{3, 1, 220, 320},
		{2, 0, 220, 460},
		{6, 0, 220, 600},
		{1, 1, 220, 740},
		{0, 0, 210, 880},
		{2, 0, 200, 1020},
		{6, 0, 270, 1160},
		{4, 1, 210, 1300},
		{0, 0, 220, 1440},
		{1, 1, 200, 1580},
		{3, 0, 200, 1720},
		{6, 0, 230, 1860},
		{2, 1, 210, 2000},
		{1, 0, 210, 2140},
		{4, 1, 210, 2280},
		{6, 0, 230, 2400},
		{1, 1, 220, 2540},
		{0, 0, 210, 2680},
		{2, 0, 220, 2800},
		{3, 0, 230, 2940},
		{4, 1, 210, 3080},
		{4, 0, 220, 3220},
		{0, 0, 210, 3360},
		{2, 0, 220, 3500},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0},
		{0, 0, 0, 0}
	},
	{
		{0, 1, 200, 200},
		{4, 1, 200, 320},
		{2, 1, 210, 440},
		{3, 0, 190, 560},
		{0, 1, 180, 680},
		{6, 0, 240, 800},
		{5, 1, 200, 920},
		{1, 1, 220, 1040},
		{0, 0, 210, 1160},
		{1, 1, 180, 1280},
		{3, 1, 200, 1400},
		{6, 0, 230, 1520},
		{0, 1, 210, 1640},
		{3, 1, 190, 1760},
		{4, 1, 180, 1880},
		{2, 0, 210, 2000},
		{1, 0, 200, 2120},
		{5, 1, 220, 2240},
		{6, 0, 300, 2360},
		{5, 1, 180, 2480},
		{4, 1, 210, 2600},
		{3, 0, 200, 2720},
		{1, 1, 220, 2840},
		{0, 1, 210, 2960},
		{6, 0, 280, 3080},
		{5, 1, 190, 3200},
		{1, 0, 180, 3320},
		{2, 1, 210, 3440},
		{3, 0, 200, 3560},
		{0, 0, 180, 3680}
	},
	{
		{1, 1, 200, 200},
		{0, 1, 200, 320},
		{5, 1, 210, 440},
		{3, 0, 190, 560},
		{4, 1, 180, 680},
		{6, 0, 240, 800},
		{5, 1, 200, 920},
		{2, 1, 220, 1040},
		{4, 1, 210, 1160},
		{0, 0, 180, 1280},
		{1, 1, 200, 1400},
		{6, 0, 230, 1520},
		{2, 1, 210, 1640},
		{3, 1, 190, 1760},
		{1, 1, 180, 1880},
		{0, 1, 210, 2000},
		{5, 0, 200, 2120},
		{2, 1, 220, 2240},
		{6, 0, 300, 2360},
		{1, 1, 180, 2480},
		{3, 1, 210, 2600},
		{5, 0, 200, 2720},
		{1, 1, 220, 2840},
		{1, 1, 210, 2960},
		{6, 0, 280, 3080},
		{5, 1, 190, 3200},
		{2, 0, 180, 3320},
		{0, 1, 210, 3440},
		{1, 0, 200, 3560},
		{3, 0, 180, 3680}
	},
	{
		{2, 1, 200, 200},
		{3, 1, 200, 320},
		{1, 0, 210, 440},
		{4, 1, 190, 560},
		{1, 1, 180, 680},
		{6, 0, 240, 800},
		{5, 1, 200, 920},
		{1, 1, 220, 1040},
		{2, 0, 210, 1160},
		{0, 1, 180, 1280},
		{5, 1, 200, 1400},
		{6, 0, 230, 1520},
		{4, 1, 210, 1640},
		{3, 1, 190, 1760},
		{1, 1, 180, 1880},
		{4, 0, 210, 2000},
		{2, 1, 200, 2120},
		{1, 1, 220, 2240},
		{6, 0, 300, 2360},
		{3, 1, 180, 2480},
		{4, 1, 210, 2600},
		{5, 1, 200, 2720},
		{1, 1, 220, 2840},
		{0, 1, 210, 2960},
		{6, 0, 280, 3080},
		{2, 1, 190, 3200},
		{3, 1, 180, 3320},
		{0, 0, 210, 3440},
		{1, 0, 200, 3560},
		{4, 0, 180, 3680}
	},
	{
		{5, 1, 200, 200},
		{4, 1, 200, 320},
		{2, 0, 210, 440},
		{3, 1, 190, 560},
		{0, 1, 180, 680},
		{6, 0, 240, 800},
		{2, 1, 200, 920},
		{1, 1, 220, 1040},
		{0, 1, 210, 1160},
		{4, 1, 180, 1280},
		{1, 1, 200, 1400},
		{6, 0, 230, 1520},
		{4, 1, 210, 1640},
		{1, 1, 190, 1760},
		{0, 1, 180, 1880},
		{2, 0, 210, 2000},
		{3, 1, 200, 2120},
		{0, 1, 220, 2240},
		{6, 0, 300, 2360},
		{3, 1, 180, 2480},
		{1, 1, 210, 2600},
		{5, 1, 200, 2720},
		{2, 0, 220, 2840},
		{0, 1, 210, 2960},
		{6, 0, 280, 3080},
		{4, 1, 190, 3200},
		{1, 1, 180, 3320},
		{0, 0, 210, 3440},
		{5, 1, 200, 3560},
		{2, 1, 180, 3680}
	},
	{
		{4, 1, 200, 200},
		{2, 1, 200, 320},
		{1, 1, 210, 440},
		{3, 1, 190, 560},
		{1, 1, 180, 680},
		{6, 0, 240, 800},
		{4, 1, 200, 920},
		{1, 1, 220, 1040},
		{0, 1, 210, 1160},
		{1, 0, 180, 1280},
		{0, 1, 200, 1400},
		{6, 0, 230, 1520},
		{0, 1, 210, 1640},
		{1, 1, 190, 1760},
		{5, 1, 180, 1880},
		{2, 1, 210, 2000},
		{3, 1, 200, 2120},
		{0, 1, 220, 2240},
		{6, 0, 300, 2360},
		{0, 1, 180, 2480},
		{1, 1, 210, 2600},
		{2, 0, 200, 2720},
		{3, 1, 220, 2840},
		{5, 1, 210, 2960},
		{6, 0, 280, 3080},
		{1, 1, 190, 3200},
		{2, 0, 180, 3320},
		{0, 1, 210, 3440},
		{1, 1, 200, 3560},
		{4, 1, 180, 3680}
	},
	{
		{0, 1, 200, 200},
		{2, 1, 200, 320},
		{3, 1, 210, 440},
		{5, 1, 190, 560},
		{1, 1, 180, 680},
		{6, 0, 240, 800},
		{4, 1, 200, 920},
		{1, 1, 220, 1040},
		{0, 1, 210, 1160},
		{3, 1, 180, 1280},
		{0, 1, 200, 1400},
		{6, 0, 230, 1520},
		{1, 1, 210, 1640},
		{2, 1, 190, 1760},
		{1, 1, 180, 1880},
		{4, 1, 210, 2000},
		{5, 1, 200, 2120},
		{0, 1, 220, 2240},
		{2, 0, 300, 2360},
		{3, 1, 180, 2480},
		{4, 1, 210, 2600},
		{0, 1, 200, 2720},
		{0, 1, 220, 2840},
		{1, 1, 210, 2960},
		{6, 0, 280, 3080},
		{2, 1, 190, 3200},
		{3, 1, 180, 3320},
		{5, 1, 210, 3440},
		{4, 1, 200, 3560},
		{1, 1, 180, 3680}
	}
};

// game level variables
char *levels[60] = 
{
"NNNMOLNNNN",
"NN@<O=ANNN",
"NNBOOOCNNN",
"NNBOOOO2NN",
"NNBOOOOOJN",
"NN0OOOOOFN",
"NNNBOOOOCN",
"NNNBOOOOCN",
"NN3OOOOO7N",
"NNBOOOOO5N",
"NNBOOOOO8N",
"NNBOOOOOCN",
"NNDOOOOOCN",
"NNHOOOOO7N",
"NNN6OOOO5N",
"NNN4OOOO5N",
"NNN4OOOO5N",
"NNN4OOOO5N",
"NNN9OOOO8N",
"NNNBOOOOCN",
"NNNBOOOOCN",
"NNKOOOOOEN",
"NNGOOOOOIN",
"NKOOOOOENN",
"NGOOOOOINN",
"NBOOOOCNNN",
"NBOOOOCNNN",
"NDOOOOCNNN",
"NHOOOOCNNN",
"NNDOOOCNNN",
"NNHOOOCNNN",
"NNNBOOCNNN",
"NNNBOOCNNN",
"NNNBOOOJNN",
"NNKOOOOFNN",
"NNGOOOOOJN",
"NNBOOOOOFN",
"NNBOOOOOCN",
"NNBOOOOOCN",
"NNBOOOOOCN",
"NNDOOOOOEN",
"NNHOOOOOIN",
"NNNBOOOENN",
"NNNBOOOINN",
"NNNBOOCNNN",
"NN3OOOCNNN",
"NKOOOOCNNN",
"N4OOOOO2NN",
"N4OOOOOOJN",
"N4OOOOOO5N",
"N4OOOOOO5N",
"NHOOOOOOFN",
"NNBOOOOOCN",
"NNBOOOOOCN",
"NNBOOOOOCN",
"NNBOOOOOCN",
"NNBOOOOOCN",
"NNBOOOO:>N",
"NN?;O:>NNN",
"NNNMOLNNNN",
};

// FUNCTIONS //////////////////////////////////////////////

// Main message handler of the system
LRESULT CALLBACK WindowProc(HWND hwnd, UINT msg, WPARAM wparam, LPARAM lparam)
{
	PAINTSTRUCT	ps;		// used in WM_PAINT
	HDC			hdc;	// handle to a device context

	// Perform actions depending on message
	switch(msg)
	{
		case WM_CREATE: 
        {
			// do initialization stuff here
			return(0);
		} break;
		
		case WM_PAINT:
		{
			// start painting
			hdc = BeginPaint(hwnd,&ps);
			
			// end painting
			EndPaint(hwnd,&ps);
			return(0);
        } break;

		case WM_DESTROY: 
		{
			// kill the application			
			PostQuitMessage(0);
			return(0);
		} break;

		default:
			break;
    } // end switch

	// process any messages that we didn't take care of 
	return (DefWindowProc(hwnd, msg, wparam, lparam));

} // end WindowProc

// WINMAIN ////////////////////////////////////////////////

int WINAPI WinMain(HINSTANCE hinstance,	HINSTANCE hprevinstance, LPSTR lpcmdline, int ncmdshow)
{
	WNDCLASS	winclass;	// this will hold the class we create
	HWND		hwnd;		// generic window handle
	MSG			msg;		// generic message
	HDC			hdc;		// generic dc
	PAINTSTRUCT ps;			// generic paintstruct

	// first fill in the window class stucture
	winclass.style			= CS_DBLCLKS | CS_OWNDC | CS_HREDRAW | CS_VREDRAW;
	winclass.lpfnWndProc	= WindowProc;
	winclass.cbClsExtra		= 0;
	winclass.cbWndExtra		= 0;
	winclass.hInstance		= hinstance;
	winclass.hIcon			= LoadIcon(NULL, IDI_APPLICATION);
	winclass.hCursor		= LoadCursor(NULL, IDC_ARROW);
	winclass.hbrBackground	= (HBRUSH)GetStockObject(BLACK_BRUSH);
	winclass.lpszMenuName	= NULL; 
	winclass.lpszClassName	= WINDOW_CLASS_NAME;

	// register the window class
	if (!RegisterClass(&winclass))
	{
		return(0);
	}

	// create the window, note the use of WS_POPUP
	if (!(hwnd = CreateWindow(WINDOW_CLASS_NAME,		// class
							  "Aqua Flyer II",			// title
							  WS_POPUP | WS_VISIBLE,	// window options
						 	  0,0,						// x,y
							  WINDOW_WIDTH,				// width
						      WINDOW_HEIGHT,			// height
							  NULL,						// handle to parent 
							  NULL,						// handle to menu
							  hinstance,				// instance
							  NULL)))					// creation parms
	{
		return(0);
	}

	// save the window handle and instance in a global
	main_window_handle = hwnd;
	main_instance      = hinstance;

	// perform all game console specific initialization
	Game_Init();

	// enter main event loop
	while(1)
	{
		if (PeekMessage(&msg,NULL,0,0,PM_REMOVE))
		{ 
			// test if this is a quit
			if (msg.message == WM_QUIT)
			{
				break;
			}
	
			// translate any accelerator keys
			TranslateMessage(&msg);

			// send the message to the window proc
			DispatchMessage(&msg);
		} // end if
    
		// main game processing goes here
		Game_Main();
	} // end while

	// shutdown game and release all resources
	Game_Shutdown();

	// return to Windows like this
	return(msg.wParam);

} // end WinMain

///////////////////////////////////////////////////////////

// WINX GAME PROGRAMMING CONSOLE FUNCTIONS ////////////////

////////////////////////////////////////////////////////////

int Game_Init(void *parms)//done
{
	// this function initializes all components of the game

	// seed random number generate
	srand(Start_Clock());

	// initialize directdraw
	DDraw_Init(SCREEN_WIDTH, SCREEN_HEIGHT, SCREEN_BPP);

	// initialize directsound
	DSound_Init();

	// initialize Directinput
	DInput_Init();

	// initialize all input devices
	DInput_Init_Keyboard();

	Init_Screens();
	Init_Sounds();
	Init_World();
	Init_Flyer();
	Init_Boats();
	Init_Cruisers();
	Init_Destroyers();
	Init_Helicopters();
	Init_Planes();
	Init_Jets();
	Init_Explosions();
	Init_Missles();
	Init_Fuel();
	Init_Bridge();
	Init_Info();

	// set clipping rectangle to screen extents so objects dont
	// mess up at edges
	RECT screen_rect = {0, 0, screen_width, screen_height};
	lpddclipper = DDraw_Attach_Clipper(lpddsback, 1, &screen_rect);

	// hide the mouse
	ShowCursor(FALSE);

	// return success
	return(1);
}

///////////////////////////////////////////////////////////

int Game_Main(void *parms)//done
{
	// this is the main function of game
	// it executes appropriate functions depending on state

	// start the timing clock
	Start_Clock();

	//call the particular handle function depending on game state
	switch(curr_game_state)
	{
		case MAIN_SCREEN:
			Handle_Main_Screen();
			break;
		case NEW_GAME_SCREEN:
			Handle_New_Game_Screen();
			break;
		case INSTRUCTION_SCREEN:
			Handle_Instruction_Screen();
			break;
		case GAME_FINISH_SCREEN:
			Handle_Game_Finish_Screen();
			break;
		case GAME_OVER:
			Handle_Game_Over_Screen();
			break;
		case GAME_PLAY:
			Handle_Game_Play();
			break;
		case GAME_RESET:
			Game_Reset();
			break;
		case NEW_GAME_INIT:
			Handle_New_Game_Init();
			break;
		case NEXT_LEVEL:
			Handle_Next_Level();
			break;
		case FLYER_EXPLODE:
			Handle_Flyer_Explode();
			break;
		case EXIT_GAME:
			Handle_Exit_Game();
			break;
		case EXIT_FADE:
			break;
	}

	// sync to 30 fps
	Wait_Clock(30);

	// return success
	return(1);
}

///////////////////////////////////////////////////////////

int Game_Shutdown(void *parms)//done
{
	// this function is where you shutdown your game and
	// release all resources that you allocated

	// delete all game graphics
	Delete_Screens();
	Delete_World();
	Delete_Flyer();
	Delete_Boats();
	Delete_Cruisers();
	Delete_Destroyers();
	Delete_Helicopters();
	Delete_Planes();
	Delete_Jets();
	Delete_Explosions();
	Delete_Missles();
	Delete_Fuel();
	Delete_Bridge();
	Delete_Info();

	// shutdown directdraw
	DDraw_Shutdown();

	// unload sounds
	DSound_Delete_All_Sounds();

	// shutdown directsound
	DSound_Shutdown();

	// release all input devices
	DInput_Release_Keyboard();

	// return success
	return(1);
}

///////////////////////////////////////////////////////////

void Game_Reset(void)//done
{
	// this function resets all the game related systems and vars for another run

	world_x = 0;
	world_y = 3391;

	flyer_fire_counter = FIRE_PAUSE_TIME;
	flyer_engine_on = 0;

	DSound_Stop_All_Sounds();

	Reset_Boats();
	Reset_Cruisers();
	Reset_Destroyers();
	Reset_Helicopters();
	Reset_Planes();
	Reset_Jets();
	Reset_Explosions();
	Reset_Missles();
	Reset_Fuel();
	Reset_Bridge();

	Start_Flyer();
	Start_Boats();
	Start_Cruisers();
	Start_Destroyers();
	Start_Helicopters();
	Start_Planes();
	Start_Jets();
	Start_Fuel();
	Start_Bridge();

	curr_game_state = GAME_PLAY;

}

///////////////////////////////////////////////////////////

void Handle_Main_Screen(void)//done
{
	// this function handles main game screen
	
	DInput_Read_Keyboard();

	// check of user is trying to exit
	if (keyboard_state[DIK_ESCAPE])
	{
		curr_game_state = EXIT_GAME;
		PostMessage(main_window_handle, WM_DESTROY,0,0);
		return;
	}

	if (keyboard_state[DIK_N])
	{
		curr_game_state = NEW_GAME_SCREEN;
		return;
	}

	if (keyboard_state[DIK_I])
	{
		curr_game_state = INSTRUCTION_SCREEN;
		return;
	}

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);

	// lock the back buffer
	DDraw_Lock_Back_Surface();

	Set_Palette(menu_pal);
	// draw the main game screen
	Draw_Bitmap(&main_screen, back_buffer, back_lpitch, 0);

	// unlock the back buffer
	DDraw_Unlock_Back_Surface();

	// flip the surfaces
	DDraw_Flip();
}

///////////////////////////////////////////////////////////

void Handle_New_Game_Screen(void)//done
{
	// this function handles new game screen
	
	DInput_Read_Keyboard();

	// check if user is trying to exit
	if (keyboard_state[DIK_BACK])
	{
		curr_game_state = MAIN_SCREEN;
		return;
	}

	// check if easy was selected
	if (keyboard_state[DIK_E])
	{
		curr_game_difficulty = EASY;
		curr_game_state = NEW_GAME_INIT;
		return;
	}

	// check if moderate was selected
	if (keyboard_state[DIK_M])
	{
		curr_game_difficulty = MODERATE;
		curr_game_state = NEW_GAME_INIT;
		return;
	}

	// check if difficult was selected
	if (keyboard_state[DIK_D])
	{
		curr_game_difficulty = DIFFICULT;
		curr_game_state = NEW_GAME_INIT;
		return;
	}

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);

	// lock the back buffer
	DDraw_Lock_Back_Surface();

	Set_Palette(menu_pal);
	// draw the new game screen
	Draw_Bitmap(&new_game_screen, back_buffer, back_lpitch, 0);

	// unlock the back buffer
	DDraw_Unlock_Back_Surface();

	// flip the surfaces
	DDraw_Flip();
}

///////////////////////////////////////////////////////////

void Handle_Instruction_Screen(void)//done
{
	// this function handles options screen
	
	DInput_Read_Keyboard();

	// check of user is trying to exit
	if (keyboard_state[DIK_BACK])
	{
		curr_game_state = MAIN_SCREEN;
		return;
	}

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);

	// lock the back buffer
	DDraw_Lock_Back_Surface();

	Set_Palette(menu_pal);
	// draw the options screen
	Draw_Bitmap(&instruction_screen, back_buffer, back_lpitch, 0);

	// unlock the back buffer
	DDraw_Unlock_Back_Surface();

	// flip the surfaces
	DDraw_Flip();
}

///////////////////////////////////////////////////////////

void Handle_Game_Finish_Screen(void)//done
{
	// this function handles game finish screen

	char score[16];					// hold score
	
	DInput_Read_Keyboard();

	// check of user is trying to exit
	if (keyboard_state[DIK_BACK])
	{
		curr_game_state = MAIN_SCREEN;
		return;
	}

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);

	// lock the back buffer
	DDraw_Lock_Back_Surface();

	Set_Palette(menu_pal);
	// draw the game finish screen
	Draw_Bitmap(&game_finish_screen, back_buffer, back_lpitch, 0);

	// unlock the back buffer
	DDraw_Unlock_Back_Surface();

	// build up scrore string
	sprintf(score, "0000000%d", flyer_score);

	// build up final string
	sprintf(buffer, "%s", &score[strlen(score)-8]);
	Draw_Text_GDI(buffer, 380-8*strlen(buffer), 340, RGB(255, 255, 255), lpddsback);

	// flip the surfaces
	DDraw_Flip();
}

///////////////////////////////////////////////////////////

void Handle_Game_Over_Screen(void)//done
{
	// this function handles game finish screen
	
	char score[16];					// hold score

	DInput_Read_Keyboard();

	// check of user is trying to exit
	if (keyboard_state[DIK_BACK])
	{
		curr_game_state = MAIN_SCREEN;
		return;
	}

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);

	// lock the back buffer
	DDraw_Lock_Back_Surface();

	Set_Palette(menu_pal);
	// draw the game finish screen
	Draw_Bitmap(&game_over_screen, back_buffer, back_lpitch, 0);

	// unlock the back buffer
	DDraw_Unlock_Back_Surface();

	// build up scrore string
	sprintf(score, "0000000%d", flyer_score);

	// build up final string
	sprintf(buffer, "%s", &score[strlen(score)-8]);
	Draw_Text_GDI(buffer, 380-8*strlen(buffer), 340, RGB(255, 255, 255), lpddsback);

	// flip the surfaces
	DDraw_Flip();
}

///////////////////////////////////////////////////////////

void Handle_Game_Play(void)//done
{

	DInput_Read_Keyboard();

	// check if user is trying to exit
	if (keyboard_state[DIK_BACK])
	{
		curr_game_state = MAIN_SCREEN;
		DSound_Stop_All_Sounds();
		return;
	}

	// check for movement right left
	if (keyboard_state[DIK_RIGHT])
	{
		flyer.curr_frame = FLYER_DIR_RIGHT;
		flyer_dx = 4;
	}
	else if (keyboard_state[DIK_LEFT])
	{
		flyer.curr_frame = FLYER_DIR_LEFT;
		flyer_dx = -4;
	}
	else
	{
		flyer.curr_frame = FLYER_DIR_NORMAL;
		flyer_dx = 0;
	}

	// check for movement up down
	if (keyboard_state[DIK_UP])
	{
		if (--flyer_dy < FLYER_SPEED_MAX)
		{
			flyer_dy = FLYER_SPEED_MAX;
		}

		// test to turn on engines
		if (!flyer_engine_on)
		{
			DSound_Play(engine_id, DSBPLAY_LOOPING);
			flyer_engine_on = 1;
		}
	}
	else if (keyboard_state[DIK_DOWN])
	{
		if (++flyer_dy > FLYER_SPEED_MIN - curr_game_difficulty)
		{
			flyer_dy = FLYER_SPEED_MIN - curr_game_difficulty;
		}
	}

	if (flyer_fire_counter < FIRE_PAUSE_TIME)
	{
		flyer_fire_counter++;
	}

	// check for user fire
	if (keyboard_state[DIK_SPACE] && (flyer_fire_counter == FIRE_PAUSE_TIME))
	{
	    // fire missle considering flyer velocity
        Fire_Missles(flyer.varsI[INDEX_WORLD_X] + 4, flyer.varsI[INDEX_WORLD_Y] - 9,
		             0, flyer_dy - MISSLE_SPEED_SLOW, MISSLE_ANIM_FLYER);
		flyer_fire_counter = 0;
		Start_Missle_Flyer_Sound(flyer.x, flyer.y, flyer.width, flyer.height);
	}

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);

	// draw the world
	Set_Palette(world_pal);
	Draw_World();

	// move all objects
	Move_Boats();
	Move_Cruisers();
	Move_Destroyers();
	Move_Helicopters();
	Move_Planes();
	Move_Jets();
	Move_Explosions();
	Move_Missles();
	Move_Fuel();
	Move_Bridge();
	Move_Flyer();

	// draw all objects
	Draw_Boats();
	Draw_Cruisers();
	Draw_Destroyers();
	Draw_Helicopters();
	Draw_Planes();
	Draw_Jets();
	Draw_Explosions();
	Draw_Missles();
	Draw_Fuel();
	Draw_Bridge();
	Draw_Flyer();
	Draw_Info();

	// flip the surfaces
	DDraw_Flip();
}

///////////////////////////////////////////////////////////

void Handle_Exit_Game(void)//DONE
{
	// this function handles game exit

	curr_game_state = EXIT_FADE;
	Screen_Transitions(SCREEN_DARKNESS, NULL, 0);
	DDraw_Fill_Surface(lpddsprimary, 0);
	DDraw_Fill_Surface(lpddsback, 0);
}

///////////////////////////////////////////////////////////

void Handle_Flyer_Explode(void)//done
{
	// this function handles flyer explosion

	// increase explosion counter
	flyer_explode++;

	// stop engine sound
	DSound_Stop_Sound(engine_id);

	// clear the drawing surface
	DDraw_Fill_Surface(lpddsback, 0);

	// destroy fired missles
	Reset_Missles();

	// draw the world
	Set_Palette(world_pal);
	Draw_World();

	// move all objects
	Move_Boats();
	Move_Cruisers();
	Move_Destroyers();
	Move_Helicopters();
	Move_Planes();
	Move_Jets();
	Move_Explosions();
	Move_Fuel();
	Move_Bridge();
	Move_Flyer();

	// draw all objects
	Draw_Boats();
	Draw_Cruisers();
	Draw_Destroyers();
	Draw_Helicopters();
	Draw_Planes();
	Draw_Jets();
	Draw_Explosions();
	Draw_Fuel();
	Draw_Bridge();
	Draw_Flyer();
	Draw_Info();

	// flip the surfaces
	DDraw_Flip();

	// end explosion state when anim finished
	if (flyer_explode >= NUM_TILES_EXPLOSION)
	{
		flyer_explode = 0;

		if (--flyer_ships < 0)
		{
			curr_game_state = GAME_OVER;
			DSound_Stop_All_Sounds();
		}
		else
		{
			curr_game_state = GAME_RESET;
		}

		flyer_fuel = 1000;

		Sleep(1000);
	}
}

///////////////////////////////////////////////////////////

void Handle_New_Game_Init(void)//done
{
	// this function intializes new game 

	curr_game_level = 0;
	flyer_fuel = 1000;
	flyer_ships = 3;
	flyer_score = 0;
	curr_game_state = GAME_RESET;
}

///////////////////////////////////////////////////////////

void Handle_Next_Level(void)//done
{
	// this function intializes next level 

	DSound_Stop_All_Sounds();

	if (curr_game_level >= MAX_LEVELS)
	{
		curr_game_state = GAME_FINISH_SCREEN;
		Screen_Transitions(SCREEN_DARKNESS, NULL, 0);
		DDraw_Fill_Surface(lpddsprimary, 0);
		DDraw_Fill_Surface(lpddsback, 0);
	}
	else
	{
		flyer_fuel = 1000;

		if (flyer_ships < 3)
		{
			flyer_ships++;
		}

		curr_game_state = GAME_RESET;
		Screen_Transitions(SCREEN_DARKNESS, NULL, 0);
		DDraw_Fill_Surface(lpddsprimary, 0);
		DDraw_Fill_Surface(lpddsback, 0);
	}
}

///////////////////////////////////////////////////////////

void Init_Boats(void)//DONE
{
	// this function initializes all patrol boats

	int i;

	// load in boat tiles
	Load_Bitmap_File(&bitmap8bit, "art/boat.bmp");

	// create the boat bob
	Create_BOB(&boats[0], 9999, 9999, BOAT_TILE_WIDTH, BOAT_TILE_HEIGHT, NUM_TILES_BOAT,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each boat bitmap into the boat BOB object
	for (i = 0; i < NUM_TILES_BOAT; i++)
	{
		Load_Frame_BOB(&boats[0], &bitmap8bit, i, i, 0, BITMAP_EXTRACT_MODE_CELL);
	}

	// set state to off
	boats[0].state = BOAT_STATE_DEAD;

	// make copies
	for (i = 1; i < BOAT_MAX; i++)
	{
		memcpy(&boats[i], &boats[0], sizeof(BOB));
	}

	// unload the boat tiles
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Boats(void)//DONE
{
	// this function sets parameters for active patrol boats

	int j = 0;
	int num_enemy = num_enemies[curr_game_level];

	for (int i = 0; i < num_enemy; i++)
	{
		if (enemies[curr_game_level][i][ENEMY_TYPE] == TYPE_BOAT)
		{
			// set state
			boats[j].state = BOAT_STATE_ALIVE;
			boats[j].counter_1 = 0;

			// set position
			boats[j].varsI[INDEX_WORLD_X] = enemies[curr_game_level][i][POS_X];
			boats[j].varsI[INDEX_WORLD_Y] = enemies[curr_game_level][i][POS_Y];
			boats[j].x = boats[j].varsI[INDEX_WORLD_X] - (boats[j].width >> 1) - world_x;
			boats[j].y = boats[j].varsI[INDEX_WORLD_Y] - (boats[j].height >> 1) - world_y;

			// set weapons
			boats[j].varsI[INDEX_WEAPON] = enemies[curr_game_level][i][WEAPON_STATE];

			// set direction
			if ((rand()%2) == 0)
			{
				// set direction to left
				boats[j].varsI[INDEX_DIR] = BOAT_DIR_LEFT;

				// set velocity
				boats[j].xv = RAND_RANGE(-BOAT_SPEED_MAX, -BOAT_SPEED_MIN);
				boats[j].yv = 0;
			}
			else
			{
				// set direction to right
				boats[j].varsI[INDEX_DIR] = BOAT_DIR_RIGHT;

				// set velocity
				boats[j].xv = RAND_RANGE(BOAT_SPEED_MIN, BOAT_SPEED_MAX);
				boats[j].yv = 0;

			}

			j++;
		}
	}
}

//////////////////////////////////////////////////////////

void Move_Boats(void)//DONE
{
	// this function moves all active patrol boats

	for (int i = 0; i < BOAT_MAX; i++)
	{
		if (boats[i].state == BOAT_STATE_ALIVE)
		{

			boats[i].varsI[INDEX_WORLD_X] += boats[i].xv;
			boats[i].x += boats[i].xv;

			// test for collision with player
			if (Collision_Test(boats[i].varsI[INDEX_WORLD_X] - (boats[i].width >> 1),
							   boats[i].varsI[INDEX_WORLD_Y] - (boats[i].height >> 1),
							   boats[i].width, boats[i].height,
							   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
							   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
							   flyer.width, flyer.height))
			{
				boats[i].state = BOAT_STATE_DEAD;
				flyer.state = FLYER_STATE_DEAD;
				Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosions(boats[i].varsI[INDEX_WORLD_X], boats[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosion_Sound();
				Start_Explosion_Sound();
				curr_game_state = FLYER_EXPLODE;
				break;
			}

			// test for collisions with flyer missles
			for (int missle = 0; missle < MISSLE_MAX; missle++)
			{
				if (missles[missle].state == MISSLE_STATE_ON && 
					missles[missle].anim_state == MISSLE_ANIM_FLYER)
				{
					// test for collision 
					if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
								       missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
									   missles[missle].width, missles[missle].height,
									   boats[i].varsI[INDEX_WORLD_X] - (boats[i].width >> 1),
									   boats[i].varsI[INDEX_WORLD_Y] - (boats[i].height >> 1),
									   boats[i].width, boats[i].height))
					{
						// kill missle
						missles[missle].state = MISSLE_STATE_OFF;
						missles[missle].counter_1 = 0;

						// kill boat
						boats[i].state = BOAT_STATE_DEAD;
						Start_Explosions(boats[i].varsI[INDEX_WORLD_X], boats[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
						Start_Explosion_Sound();

						flyer_score += BOAT_POINTS;
						break;
					}
				}
			}

			DDraw_Lock_Back_Surface();

			// test for collision with river bank
			if (Color_Scan(boats[i].x, boats[i].y,
						   boats[i].x + boats[i].width, boats[i].y + boats[i].height,
						   COLOR_RIVER_BANK, COLOR_RIVER_BANK_2, back_buffer, back_lpitch))
			{
				// back the boat up along its last trajectory
				boats[i].varsI[INDEX_WORLD_X] -= 5 * boats[i].xv;
				boats[i].xv *= -1;

				// change direction
				if (boats[i].varsI[INDEX_DIR] == BOAT_DIR_LEFT)
				{
					boats[i].varsI[INDEX_DIR] = BOAT_DIR_RIGHT;
				}
				else
				{
					boats[i].varsI[INDEX_DIR] = BOAT_DIR_LEFT;
				}
			}

			DDraw_Unlock_Back_Surface();

			// fire weapon if equipped
			if (boats[i].varsI[INDEX_WEAPON] == BOAT_WEAPON_ON)
			{
				if (++boats[i].counter_1 > 30)
				{
					int speed;

					if (boats[i].varsI[INDEX_DIR] == BOAT_DIR_LEFT)
					{
						speed = boats[i].xv - 5;
					}
					else
					{
						speed = boats[i].xv + 5;
					}

					Fire_Missles(boats[i].varsI[INDEX_WORLD_X], boats[i].varsI[INDEX_WORLD_Y],
								 speed, 0, MISSLE_ANIM_ENEMY);
					Start_Missle_Enemy_Sound(boats[i].x, boats[i].y, boats[i].width, boats[i].height);
					boats[i].counter_1 = 0;
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Boats(void)//DONE
{
	// this function draws all active patrol boats

	for (int i = 0; i < BOAT_MAX; i++)
	{
		if (boats[i].state == BOAT_STATE_ALIVE)
		{
			// transform to screen coords
			boats[i].x = boats[i].varsI[INDEX_WORLD_X] - (boats[i].width >> 1) - world_x;
			boats[i].y = boats[i].varsI[INDEX_WORLD_Y] - (boats[i].height >> 1) - world_y;

			// set correct frame
			boats[i].curr_frame = boats[i].varsI[INDEX_DIR];

			// draw patrol boat
			Draw_BOB(&boats[i], lpddsback);
		}
	}
}

///////////////////////////////////////////////////////////

void Reset_Boats(void)//DONE
{
	// this function resets all patrol boats

	for (int i = 0; i < BOAT_MAX; i++)
    {
		boats[i].state = BOAT_STATE_DEAD;
    }
}

////////////////////////////////////////////////////////////

void Delete_Boats(void)//DONE
{
	// this function deletes all patrol boats data

	for (int i = 0; i < BOAT_MAX; i++)
	{
		Destroy_BOB(&boats[i]);
	}
}

///////////////////////////////////////////////////////////

void Init_Cruisers(void)//DONE
{
	// this function initializes all cruisers

	int i;

	// load in boat tiles
	Load_Bitmap_File(&bitmap8bit, "art/cruiser.bmp");

	// create the boat bob
	Create_BOB(&cruisers[0], 9999, 9999, CRUISER_TILE_WIDTH, CRUISER_TILE_HEIGHT, NUM_TILES_CRUISER,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each boat bitmap into the boat BOB object
	for (i = 0; i < NUM_TILES_CRUISER; i++)
	{
		Load_Frame_BOB(&cruisers[0], &bitmap8bit, i, i, 0, BITMAP_EXTRACT_MODE_CELL);
	}

	// set state to off
	cruisers[0].state = CRUISER_STATE_DEAD;

	// make copies
	for (i = 1; i < CRUISER_MAX; i++)
	{
		memcpy(&cruisers[i], &cruisers[0], sizeof(BOB));
	}

	// unload the boat tiles
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Cruisers(void)//DONE
{
	// this function sets parameters for active cruisers

	int j = 0;
	int num_enemy = num_enemies[curr_game_level];

	for (int i = 0; i < num_enemy; i++)
	{
		if (enemies[curr_game_level][i][ENEMY_TYPE] == TYPE_CRUISER)
		{
			// set state
			cruisers[j].state = CRUISER_STATE_ALIVE;
			cruisers[j].counter_1 = 0;

			// set position
			cruisers[j].varsI[INDEX_WORLD_X] = enemies[curr_game_level][i][POS_X];
			cruisers[j].varsI[INDEX_WORLD_Y] = enemies[curr_game_level][i][POS_Y];
			cruisers[j].x = cruisers[j].varsI[INDEX_WORLD_X] - (cruisers[j].width >> 1) - world_x;
			cruisers[j].y = cruisers[j].varsI[INDEX_WORLD_Y] - (cruisers[j].height >> 1) - world_y;

			// set weapons
			cruisers[j].varsI[INDEX_WEAPON] = enemies[curr_game_level][i][WEAPON_STATE];

			// set direction
			if ((rand()%2) == 0)
			{
				// set direction to left
				cruisers[j].varsI[INDEX_DIR] = CRUISER_DIR_LEFT;

				// set velocity
				cruisers[j].xv = RAND_RANGE(-CRUISER_SPEED_MAX, -CRUISER_SPEED_MIN);
				cruisers[j].yv = 0;
			}
			else
			{
				// set direction to right
				cruisers[j].varsI[INDEX_DIR] = CRUISER_DIR_RIGHT;

				// set velocity
				cruisers[j].xv = RAND_RANGE(CRUISER_SPEED_MIN, CRUISER_SPEED_MAX);
				cruisers[j].yv = 0;

			}

			j++;
		}
	}
}

//////////////////////////////////////////////////////////

void Move_Cruisers(void)//DONE
{
	// this function moves all active cruisers

	for (int i = 0; i < CRUISER_MAX; i++)
	{
		if (cruisers[i].state == CRUISER_STATE_ALIVE)
		{

			cruisers[i].varsI[INDEX_WORLD_X] += cruisers[i].xv;
			cruisers[i].x += cruisers[i].xv;

			// test for collision with player
			if (Collision_Test(cruisers[i].varsI[INDEX_WORLD_X] - (cruisers[i].width >> 1),
							   cruisers[i].varsI[INDEX_WORLD_Y] - (cruisers[i].height >> 1),
							   cruisers[i].width, cruisers[i].height,
							   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
							   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
							   flyer.width, flyer.height))
			{
				cruisers[i].state = CRUISER_STATE_DEAD;
				flyer.state = FLYER_STATE_DEAD;
				Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosions(cruisers[i].varsI[INDEX_WORLD_X], cruisers[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosion_Sound();
				Start_Explosion_Sound();
				curr_game_state = FLYER_EXPLODE;
				break;
			}

			// test for collisions with flyer missles
			for (int missle = 0; missle < MISSLE_MAX; missle++)
			{
				if (missles[missle].state == MISSLE_STATE_ON && 
					missles[missle].anim_state == MISSLE_ANIM_FLYER)
				{
					// test for collision 
					if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
								       missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
									   missles[missle].width, missles[missle].height,
									   cruisers[i].varsI[INDEX_WORLD_X] - (cruisers[i].width >> 1),
									   cruisers[i].varsI[INDEX_WORLD_Y] - (cruisers[i].height >> 1),
									   cruisers[i].width, cruisers[i].height))
					{
						// kill missle
						missles[missle].state     = MISSLE_STATE_OFF;
						missles[missle].counter_1 = 0;

						// kill boat
						cruisers[i].state = CRUISER_STATE_DEAD;
						Start_Explosions(cruisers[i].varsI[INDEX_WORLD_X], cruisers[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
						Start_Explosion_Sound();

						flyer_score += CRUISER_POINTS;
						break;
					}
				}
			}

			DDraw_Lock_Back_Surface();

			// test for collision with river bank
			if (Color_Scan(cruisers[i].x, cruisers[i].y,
						   cruisers[i].x + cruisers[i].width, cruisers[i].y + cruisers[i].height,
						   COLOR_RIVER_BANK, COLOR_RIVER_BANK_2, back_buffer, back_lpitch))
			{
				// back the boat up along its last trajectory
				cruisers[i].varsI[INDEX_WORLD_X] -= 5 * cruisers[i].xv;
				cruisers[i].xv *= -1;

				// change direction
				if (cruisers[i].varsI[INDEX_DIR] == CRUISER_DIR_LEFT)
				{
					cruisers[i].varsI[INDEX_DIR] = CRUISER_DIR_RIGHT;
				}
				else
				{
					cruisers[i].varsI[INDEX_DIR] = CRUISER_DIR_LEFT;
				}
			}

			DDraw_Unlock_Back_Surface();

			// fire weapon if equipped
			if (cruisers[i].varsI[INDEX_WEAPON] == CRUISER_WEAPON_ON)
			{
				if (++cruisers[i].counter_1 > 30)
				{
					int speed;

					if (cruisers[i].varsI[INDEX_DIR] == CRUISER_DIR_LEFT)
					{
						speed = cruisers[i].xv - 5;
					}
					else
					{
						speed = cruisers[i].xv + 5;
					}

					Fire_Missles(cruisers[i].varsI[INDEX_WORLD_X], cruisers[i].varsI[INDEX_WORLD_Y],
								 speed, 0, MISSLE_ANIM_ENEMY);
					Start_Missle_Enemy_Sound(cruisers[i].x, cruisers[i].y, cruisers[i].width, cruisers[i].height);
					cruisers[i].counter_1 = 0;
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Cruisers(void)//DONE
{
	// this function draws all active cruisers

	for (int i = 0; i < CRUISER_MAX; i++)
	{
		if (cruisers[i].state == CRUISER_STATE_ALIVE)
		{
			// transform to screen coords
			cruisers[i].x = cruisers[i].varsI[INDEX_WORLD_X] - (cruisers[i].width >> 1) - world_x;
			cruisers[i].y = cruisers[i].varsI[INDEX_WORLD_Y] - (cruisers[i].height >> 1) - world_y;

			// set correct frame
			cruisers[i].curr_frame = cruisers[i].varsI[INDEX_DIR];

			// draw patrol boat
			Draw_BOB(&cruisers[i], lpddsback);
		}
	}
}

///////////////////////////////////////////////////////////

void Reset_Cruisers(void)//DONE
{
	// this function resets all cruisers

	for (int i = 0; i < CRUISER_MAX; i++)
    {
		cruisers[i].state = CRUISER_STATE_DEAD;
    }
}

////////////////////////////////////////////////////////////

void Delete_Cruisers(void)//DONE
{
	// this function deletes all cruisers data

	for (int i = 0; i < CRUISER_MAX; i++)
	{
		Destroy_BOB(&cruisers[i]);
	}
}

///////////////////////////////////////////////////////////

void Init_Destroyers(void)//DONE
{
	// this function initializes all destroyers

	int i;

	// load in boat tiles
	Load_Bitmap_File(&bitmap8bit, "art/destroyer.bmp");

	// create the boat bob
	Create_BOB(&destroyers[0], 9999, 9999, DESTROYER_TILE_WIDTH, DESTROYER_TILE_HEIGHT, NUM_TILES_DESTROYER,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each boat bitmap into the boat BOB object
	for (i = 0; i < NUM_TILES_DESTROYER; i++)
	{
		Load_Frame_BOB(&destroyers[0], &bitmap8bit, i, i, 0, BITMAP_EXTRACT_MODE_CELL);
	}

	// set state to off
	destroyers[0].state = DESTROYER_STATE_DEAD;

	// make copies
	for (i = 1; i < DESTROYER_MAX; i++)
	{
		memcpy(&destroyers[i], &destroyers[0], sizeof(BOB));
	}

	// unload the boat tiles
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Destroyers(void)//DONE
{
	// this function sets parameters for active destroyers

	int j = 0;
	int num_enemy = num_enemies[curr_game_level];

	for (int i = 0; i < num_enemy; i++)
	{
		if (enemies[curr_game_level][i][ENEMY_TYPE] == TYPE_DESTROYER)
		{
			// set state
			destroyers[j].state = DESTROYER_STATE_ALIVE;
			destroyers[j].counter_1 = 0;

			// set position
			destroyers[j].varsI[INDEX_WORLD_X] = enemies[curr_game_level][i][POS_X];
			destroyers[j].varsI[INDEX_WORLD_Y] = enemies[curr_game_level][i][POS_Y];
			destroyers[j].x = destroyers[j].varsI[INDEX_WORLD_X] - (destroyers[j].width >> 1) - world_x;
			destroyers[j].y = destroyers[j].varsI[INDEX_WORLD_Y] - (destroyers[j].height >> 1) - world_y;

			// set weapons
			destroyers[j].varsI[INDEX_WEAPON] = enemies[curr_game_level][i][WEAPON_STATE];

			// set direction
			if ((rand()%2) == 0)
			{
				// set direction to left
				destroyers[j].varsI[INDEX_DIR] = DESTROYER_DIR_LEFT;

				// set velocity
				destroyers[j].xv = RAND_RANGE(-DESTROYER_SPEED_MAX, -DESTROYER_SPEED_MIN);
				destroyers[j].yv = 0;
			}
			else
			{
				// set direction to right
				destroyers[j].varsI[INDEX_DIR] = DESTROYER_DIR_RIGHT;

				// set velocity
				destroyers[j].xv = RAND_RANGE(DESTROYER_SPEED_MIN, DESTROYER_SPEED_MAX);
				destroyers[j].yv = 0;

			}

			j++;
		}
	}
}

//////////////////////////////////////////////////////////

void Move_Destroyers(void)//DONE
{
	// this function moves all active destroyers

	for (int i = 0; i < DESTROYER_MAX; i++)
	{
		if (destroyers[i].state == DESTROYER_STATE_ALIVE)
		{

			destroyers[i].varsI[INDEX_WORLD_X] += destroyers[i].xv;
			destroyers[i].x += destroyers[i].xv;

			// test for collision with player
			if (Collision_Test(destroyers[i].varsI[INDEX_WORLD_X] - (destroyers[i].width >> 1),
							   destroyers[i].varsI[INDEX_WORLD_Y] - (destroyers[i].height >> 1),
							   destroyers[i].width, destroyers[i].height,
							   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
							   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
							   flyer.width, flyer.height))
			{
				destroyers[i].state = DESTROYER_STATE_DEAD;
				flyer.state = FLYER_STATE_DEAD;
				Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosions(destroyers[i].varsI[INDEX_WORLD_X], destroyers[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosion_Sound();
				Start_Explosion_Sound();
				curr_game_state = FLYER_EXPLODE;
				break;
			}

			// test for collisions with flyer missles
			for (int missle = 0; missle < MISSLE_MAX; missle++)
			{
				if (missles[missle].state == MISSLE_STATE_ON && 
					missles[missle].anim_state == MISSLE_ANIM_FLYER)
				{
					// test for collision 
					if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
								       missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
									   missles[missle].width, missles[missle].height,
									   destroyers[i].varsI[INDEX_WORLD_X] - (destroyers[i].width >> 1),
									   destroyers[i].varsI[INDEX_WORLD_Y] - (destroyers[i].height >> 1),
									   destroyers[i].width, destroyers[i].height))
					{
						// kill missle
						missles[missle].state     = MISSLE_STATE_OFF;
						missles[missle].counter_1 = 0;

						// kill boat
						destroyers[i].state = DESTROYER_STATE_DEAD;
						Start_Explosions(destroyers[i].varsI[INDEX_WORLD_X], destroyers[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
						Start_Explosion_Sound();

						flyer_score += DESTROYER_POINTS;
						break;
					}
				}
			}

			DDraw_Lock_Back_Surface();

			// test for collision with river bank
			if (Color_Scan(destroyers[i].x, destroyers[i].y,
						   destroyers[i].x + destroyers[i].width, destroyers[i].y + destroyers[i].height,
						   COLOR_RIVER_BANK, COLOR_RIVER_BANK_2, back_buffer, back_lpitch))
			{
				// back the boat up along its last trajectory
				destroyers[i].varsI[INDEX_WORLD_X] -= 5 * destroyers[i].xv;
				destroyers[i].xv *= -1;

				// change direction
				if (destroyers[i].varsI[INDEX_DIR] == DESTROYER_DIR_LEFT)
				{
					destroyers[i].varsI[INDEX_DIR] = DESTROYER_DIR_RIGHT;
				}
				else
				{
					destroyers[i].varsI[INDEX_DIR] = DESTROYER_DIR_LEFT;
				}
			}

			DDraw_Unlock_Back_Surface();

			// fire weapon if equipped
			if (destroyers[i].varsI[INDEX_WEAPON] == DESTROYER_WEAPON_ON)
			{
				if (++destroyers[i].counter_1 > 30)
				{
					int speed;

					if (destroyers[i].varsI[INDEX_DIR] == DESTROYER_DIR_LEFT)
					{
						speed = destroyers[i].xv - 5;
					}
					else
					{
						speed = destroyers[i].xv + 5;
					}

					Fire_Missles(destroyers[i].varsI[INDEX_WORLD_X], destroyers[i].varsI[INDEX_WORLD_Y],
								 speed, 0, MISSLE_ANIM_ENEMY);
					Start_Missle_Enemy_Sound(destroyers[i].x, destroyers[i].y, destroyers[i].width, destroyers[i].height);
					destroyers[i].counter_1 = 0;
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Destroyers(void)//DONE
{
	// this function draws all active destroyers

	for (int i = 0; i < DESTROYER_MAX; i++)
	{
		if (destroyers[i].state == DESTROYER_STATE_ALIVE)
		{
			// transform to screen coords
			destroyers[i].x = destroyers[i].varsI[INDEX_WORLD_X] - (destroyers[i].width >> 1) - world_x;
			destroyers[i].y = destroyers[i].varsI[INDEX_WORLD_Y] - (destroyers[i].height >> 1) - world_y;

			// set correct frame
			destroyers[i].curr_frame = destroyers[i].varsI[INDEX_DIR];

			// draw patrol boat
			Draw_BOB(&destroyers[i], lpddsback);
		}
	}
}

///////////////////////////////////////////////////////////

void Reset_Destroyers(void)//DONE
{
	// this function resets all destroyers

	for (int i = 0; i < DESTROYER_MAX; i++)
    {
		destroyers[i].state = DESTROYER_STATE_DEAD;
    }
}

////////////////////////////////////////////////////////////

void Delete_Destroyers(void)//DONE
{
	// this function deletes all destroyers data

	for (int i = 0; i < DESTROYER_MAX; i++)
	{
		Destroy_BOB(&destroyers[i]);
	}
}

///////////////////////////////////////////////////////////

void Init_Helicopters(void)//DONE
{
	// this function initializes all helicopters

	int i;

	// load in boat tiles
	Load_Bitmap_File(&bitmap8bit, "art/helicopter.bmp");

	// create the boat bob
	Create_BOB(&helicopters[0], 9999, 9999, HELICOPTER_TILE_WIDTH, HELICOPTER_TILE_HEIGHT, NUM_TILES_HELICOPTER,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each boat bitmap into the boat BOB object
	for (i = 0; i < NUM_TILES_HELICOPTER; i++)
	{
		Load_Frame_BOB(&helicopters[0], &bitmap8bit, i, i, 0, BITMAP_EXTRACT_MODE_CELL);
	}

	// set state to off
	helicopters[0].state = HELICOPTER_STATE_DEAD;

	// make copies
	for (i = 1; i < HELICOPTER_MAX; i++)
	{
		memcpy(&helicopters[i], &helicopters[0], sizeof(BOB));
	}

	// unload the boat tiles
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Helicopters(void)//DONE
{
	// this function sets parameters for active helicopters

	int j = 0;
	int num_enemy = num_enemies[curr_game_level];

	for (int i = 0; i < num_enemy; i++)
	{
		if (enemies[curr_game_level][i][ENEMY_TYPE] == TYPE_HELICOPTER)
		{
			// set state
			helicopters[j].state = HELICOPTER_STATE_ALIVE;
			helicopters[j].counter_1 = 0;

			// set position
			helicopters[j].varsI[INDEX_WORLD_X] = enemies[curr_game_level][i][POS_X];
			helicopters[j].varsI[INDEX_WORLD_Y] = enemies[curr_game_level][i][POS_Y];
			helicopters[j].x = helicopters[j].varsI[INDEX_WORLD_X] - (helicopters[j].width >> 1) - world_x;
			helicopters[j].y = helicopters[j].varsI[INDEX_WORLD_Y] - (helicopters[j].height >> 1) - world_y;

			// set weapons
			helicopters[j].varsI[INDEX_WEAPON] = enemies[curr_game_level][i][WEAPON_STATE];

			// set direction
			if ((rand()%2) == 0)
			{
				// set direction to left
				helicopters[j].varsI[INDEX_DIR] = HELICOPTER_DIR_LEFT;

				// set velocity
				helicopters[j].xv = RAND_RANGE(-HELICOPTER_SPEED_MAX, -HELICOPTER_SPEED_MIN);
				helicopters[j].yv = 0;
			}
			else
			{
				// set direction to right
				helicopters[j].varsI[INDEX_DIR] = HELICOPTER_DIR_RIGHT;

				// set velocity
				helicopters[j].xv = RAND_RANGE(HELICOPTER_SPEED_MIN, HELICOPTER_SPEED_MAX);
				helicopters[j].yv = 0;

			}

			j++;
		}
	}
}

//////////////////////////////////////////////////////////

void Move_Helicopters(void)//DONE
{
	// this function moves all active helicopters

	for (int i = 0; i < HELICOPTER_MAX; i++)
	{
		if (helicopters[i].state == HELICOPTER_STATE_ALIVE)
		{

			helicopters[i].varsI[INDEX_WORLD_X] += helicopters[i].xv;
			helicopters[i].x += helicopters[i].xv;

			// test for collision with player
			if (Collision_Test(helicopters[i].varsI[INDEX_WORLD_X] - (helicopters[i].width >> 1),
							   helicopters[i].varsI[INDEX_WORLD_Y] - (helicopters[i].height >> 1),
							   helicopters[i].width, helicopters[i].height,
							   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
							   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
							   flyer.width, flyer.height))
			{
				helicopters[i].state = HELICOPTER_STATE_DEAD;
				flyer.state = FLYER_STATE_DEAD;
				Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosions(helicopters[i].varsI[INDEX_WORLD_X], helicopters[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosion_Sound();
				Start_Explosion_Sound();
				
				curr_game_state = FLYER_EXPLODE;
				break;
			}

			// test for collisions with flyer missles
			for (int missle = 0; missle < MISSLE_MAX; missle++)
			{
				if (missles[missle].state == MISSLE_STATE_ON && 
					missles[missle].anim_state == MISSLE_ANIM_FLYER)
				{
					// test for collision 
					if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
								       missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
									   missles[missle].width, missles[missle].height,
									   helicopters[i].varsI[INDEX_WORLD_X] - (helicopters[i].width >> 1),
									   helicopters[i].varsI[INDEX_WORLD_Y] - (helicopters[i].height >> 1),
									   helicopters[i].width, helicopters[i].height))
					{
						// kill missle
						missles[missle].state     = MISSLE_STATE_OFF;
						missles[missle].counter_1 = 0;

						// kill boat
						helicopters[i].state = HELICOPTER_STATE_DEAD;
						Start_Explosions(helicopters[i].varsI[INDEX_WORLD_X], helicopters[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
						Start_Explosion_Sound();

						flyer_score += HELICOPTER_POINTS;
						break;
					}
				}
			}

			DDraw_Lock_Back_Surface();

			// test for collision with river bank
			if (Color_Scan(helicopters[i].x, helicopters[i].y,
						   helicopters[i].x + helicopters[i].width, helicopters[i].y + helicopters[i].height,
						   COLOR_RIVER_BANK, COLOR_RIVER_BANK_2, back_buffer, back_lpitch))
			{
				// back the boat up along its last trajectory
				helicopters[i].varsI[INDEX_WORLD_X] -= 5 * helicopters[i].xv;
				helicopters[i].xv *= -1;

				// change direction
				if (helicopters[i].varsI[INDEX_DIR] == HELICOPTER_DIR_LEFT)
				{
					helicopters[i].varsI[INDEX_DIR] = HELICOPTER_DIR_RIGHT;
				}
				else
				{
					helicopters[i].varsI[INDEX_DIR] = HELICOPTER_DIR_LEFT;
				}

			}

			DDraw_Unlock_Back_Surface();

			// fire weapon if equipped
			if (helicopters[i].varsI[INDEX_WEAPON] == HELICOPTER_WEAPON_ON)
			{
				if (++helicopters[i].counter_1 > 30)
				{
					int speed;

					if (helicopters[i].varsI[INDEX_DIR] == HELICOPTER_DIR_LEFT)
					{
						speed = helicopters[i].xv - 5;
					}
					else
					{
						speed = helicopters[i].xv + 5;
					}

					Fire_Missles(helicopters[i].varsI[INDEX_WORLD_X], helicopters[i].varsI[INDEX_WORLD_Y],
								 speed, 0, MISSLE_ANIM_ENEMY);
					Start_Missle_Enemy_Sound(helicopters[i].x, helicopters[i].y, helicopters[i].width, helicopters[i].height);
					helicopters[i].counter_1 = 0;
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Helicopters(void)//DONE
{
	// this function draws all active helicopters

	for (int i = 0; i < HELICOPTER_MAX; i++)
	{
		if (helicopters[i].state == HELICOPTER_STATE_ALIVE)
		{
			// transform to screen coords
			helicopters[i].x = helicopters[i].varsI[INDEX_WORLD_X] - (helicopters[i].width >> 1) - world_x;
			helicopters[i].y = helicopters[i].varsI[INDEX_WORLD_Y] - (helicopters[i].height >> 1) - world_y;

			// set correct frame
			helicopters[i].curr_frame = helicopters[i].varsI[INDEX_DIR];

			// draw patrol boat
			Draw_BOB(&helicopters[i], lpddsback);
		}
	}
}

///////////////////////////////////////////////////////////

void Reset_Helicopters(void)//DONE
{
	// this function resets all helicopters

	for (int i = 0; i < HELICOPTER_MAX; i++)
    {
		helicopters[i].state = HELICOPTER_STATE_DEAD;
    }
}

////////////////////////////////////////////////////////////

void Delete_Helicopters(void)//DONE
{
	// this function deletes all helicopters data

	for (int i = 0; i < HELICOPTER_MAX; i++)
	{
		Destroy_BOB(&helicopters[i]);
	}
}

///////////////////////////////////////////////////////////

void Init_Planes(void)//DONE
{
	// this function initializes all planes

	int i;

	// load in boat tiles
	Load_Bitmap_File(&bitmap8bit, "art/plane.bmp");

	// create the boat bob
	Create_BOB(&planes[0], 9999, 9999, PLANE_TILE_WIDTH, PLANE_TILE_HEIGHT, NUM_TILES_PLANE,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each boat bitmap into the boat BOB object
	for (i = 0; i < NUM_TILES_PLANE; i++)
	{
		Load_Frame_BOB(&planes[0], &bitmap8bit, i, i, 0, BITMAP_EXTRACT_MODE_CELL);
	}

	// set state to off
	planes[0].state = PLANE_STATE_DEAD;

	// make copies
	for (i = 1; i < PLANE_MAX; i++)
	{
		memcpy(&planes[i], &planes[0], sizeof(BOB));
	}

	// unload the boat tiles
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Planes(void)//DONE
{
	// this function sets parameters for active planes

	int j = 0;
	int num_enemy = num_enemies[curr_game_level];

	for (int i = 0; i < num_enemy; i++)
	{
		if (enemies[curr_game_level][i][ENEMY_TYPE] == TYPE_PLANE)
		{
			// set state
			planes[j].state = PLANE_STATE_ALIVE;
			planes[j].counter_1 = 0;

			// set position
			planes[j].varsI[INDEX_WORLD_X] = enemies[curr_game_level][i][POS_X];
			planes[j].varsI[INDEX_WORLD_Y] = enemies[curr_game_level][i][POS_Y];
			planes[j].x = planes[j].varsI[INDEX_WORLD_X] - (planes[j].width >> 1) - world_x;
			planes[j].y = planes[j].varsI[INDEX_WORLD_Y] - (planes[j].height >> 1) - world_y;

			// set weapons
			planes[j].varsI[INDEX_WEAPON] = enemies[curr_game_level][i][WEAPON_STATE];

			// set direction
			if ((rand()%2) == 0)
			{
				// set direction to left
				planes[j].varsI[INDEX_DIR] = PLANE_DIR_LEFT;

				// set velocity
				planes[j].xv = RAND_RANGE(-PLANE_SPEED_MAX, -PLANE_SPEED_MIN);
				planes[j].yv = 0;
			}
			else
			{
				// set direction to right
				planes[j].varsI[INDEX_DIR] = PLANE_DIR_RIGHT;

				// set velocity
				planes[j].xv = RAND_RANGE(PLANE_SPEED_MIN, PLANE_SPEED_MAX);
				planes[j].yv = 0;

			}

			j++;
		}
	}
}

//////////////////////////////////////////////////////////

void Move_Planes(void)//DONE
{
	// this function moves all active planes

	for (int i = 0; i < PLANE_MAX; i++)
	{
		if (planes[i].state == PLANE_STATE_ALIVE)
		{

			planes[i].varsI[INDEX_WORLD_X] += planes[i].xv;
			planes[i].x += planes[i].xv;

			// test for collision with player
			if (Collision_Test(planes[i].varsI[INDEX_WORLD_X] - (planes[i].width >> 1),
							   planes[i].varsI[INDEX_WORLD_Y] - (planes[i].height >> 1),
							   planes[i].width, planes[i].height,
							   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
							   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
							   flyer.width, flyer.height))
			{
				planes[i].state = PLANE_STATE_DEAD;
				flyer.state = FLYER_STATE_DEAD;
				Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosions(planes[i].varsI[INDEX_WORLD_X], planes[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosion_Sound();
				Start_Explosion_Sound();

				curr_game_state = FLYER_EXPLODE;
				break;
			}

			// test for collisions with flyer missles
			for (int missle = 0; missle < MISSLE_MAX; missle++)
			{
				if (missles[missle].state == MISSLE_STATE_ON && 
					missles[missle].anim_state == MISSLE_ANIM_FLYER)
				{
					// test for collision 
					if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
								       missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
									   missles[missle].width, missles[missle].height,
									   planes[i].varsI[INDEX_WORLD_X] - (planes[i].width >> 1),
									   planes[i].varsI[INDEX_WORLD_Y] - (planes[i].height >> 1),
									   planes[i].width, planes[i].height))
					{
						// kill missle
						missles[missle].state     = MISSLE_STATE_OFF;
						missles[missle].counter_1 = 0;

						// kill boat
						planes[i].state = PLANE_STATE_DEAD;
						Start_Explosions(planes[i].varsI[INDEX_WORLD_X], planes[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
						Start_Explosion_Sound();

						flyer_score += PLANE_POINTS;
						break;
					}
				}
			}

			DDraw_Lock_Back_Surface();

			// test for collision with river bank
			if (Color_Scan(planes[i].x, planes[i].y,
						   planes[i].x + planes[i].width, planes[i].y + planes[i].height,
						   COLOR_RIVER_BANK, COLOR_RIVER_BANK_2, back_buffer, back_lpitch))
			{
				// back the boat up along its last trajectory
				planes[i].varsI[INDEX_WORLD_X] -= 5 * planes[i].xv;
				planes[i].xv *= -1;

				// change direction
				if (planes[i].varsI[INDEX_DIR] == PLANE_DIR_LEFT)
				{
					planes[i].varsI[INDEX_DIR] = PLANE_DIR_RIGHT;
				}
				else
				{
					planes[i].varsI[INDEX_DIR] = PLANE_DIR_LEFT;
				}
			}

			DDraw_Unlock_Back_Surface();

			// fire weapon if equipped
			if (planes[i].varsI[INDEX_WEAPON] == PLANE_WEAPON_ON)
			{
				if (++planes[i].counter_1 > 30)
				{
					int speed;

					if (planes[i].varsI[INDEX_DIR] == PLANE_DIR_LEFT)
					{
						speed = planes[i].xv - 5;
					}
					else
					{
						speed = planes[i].xv + 5;
					}

					Fire_Missles(planes[i].varsI[INDEX_WORLD_X], planes[i].varsI[INDEX_WORLD_Y],
								 speed, 0, MISSLE_ANIM_ENEMY);
					Start_Missle_Enemy_Sound(planes[i].x, planes[i].y, planes[i].width, planes[i].height);
					planes[i].counter_1 = 0;
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Planes(void)//DONE
{
	// this function draws all active planes

	for (int i = 0; i < PLANE_MAX; i++)
	{
		if (planes[i].state == PLANE_STATE_ALIVE)
		{
			// transform to screen coords
			planes[i].x = planes[i].varsI[INDEX_WORLD_X] - (planes[i].width >> 1) - world_x;
			planes[i].y = planes[i].varsI[INDEX_WORLD_Y] - (planes[i].height >> 1) - world_y;

			// set correct frame
			planes[i].curr_frame = planes[i].varsI[INDEX_DIR];

			// draw patrol boat
			Draw_BOB(&planes[i], lpddsback);
		}
	}
}

///////////////////////////////////////////////////////////

void Reset_Planes(void)//DONE
{
	// this function resets all planes

	for (int i = 0; i < PLANE_MAX; i++)
    {
		planes[i].state = PLANE_STATE_DEAD;
    }
}

////////////////////////////////////////////////////////////

void Delete_Planes(void)//DONE
{
	// this function deletes all planes data

	for (int i = 0; i < PLANE_MAX; i++)
	{
		Destroy_BOB(&planes[i]);
	}
}

///////////////////////////////////////////////////////////

void Init_Jets(void)//DONE
{
	// this function initializes all jets

	int i;

	// load in boat tiles
	Load_Bitmap_File(&bitmap8bit, "art/jet.bmp");

	// create the boat bob
	Create_BOB(&jets[0], 9999, 9999, JET_TILE_WIDTH, JET_TILE_HEIGHT, NUM_TILES_JET,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each boat bitmap into the boat BOB object
	for (i = 0; i < NUM_TILES_JET; i++)
	{
		Load_Frame_BOB(&jets[0], &bitmap8bit, i, i, 0, BITMAP_EXTRACT_MODE_CELL);
	}

	// set state to off
	jets[0].state = JET_STATE_DEAD;

	// make copies
	for (i = 1; i < JET_MAX; i++)
	{
		memcpy(&jets[i], &jets[0], sizeof(BOB));
	}

	// unload the boat tiles
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Jets(void)//DONE
{
	// this function sets parameters for active jets

	int j = 0;
	int num_enemy = num_enemies[curr_game_level];

	for (int i = 0; i < num_enemy; i++)
	{
		if (enemies[curr_game_level][i][ENEMY_TYPE] == TYPE_JET)
		{
			// set state
			jets[j].state = JET_STATE_ALIVE;
			jets[j].counter_1 = 0;

			// set position
			jets[j].varsI[INDEX_WORLD_X] = enemies[curr_game_level][i][POS_X];
			jets[j].varsI[INDEX_WORLD_Y] = enemies[curr_game_level][i][POS_Y];
			jets[j].x = jets[j].varsI[INDEX_WORLD_X] - (jets[j].width >> 1) - world_x;
			jets[j].y = jets[j].varsI[INDEX_WORLD_Y] - (jets[j].height >> 1) - world_y;

			// set weapons
			jets[j].varsI[INDEX_WEAPON] = enemies[curr_game_level][i][WEAPON_STATE];

			// set direction
			if ((rand()%2) == 0)
			{
				// set direction to left
				jets[j].varsI[INDEX_DIR] = JET_DIR_LEFT;

				// set velocity
				jets[j].xv = RAND_RANGE(-JET_SPEED_MAX, -JET_SPEED_MIN);
				jets[j].yv = 0;
			}
			else
			{
				// set direction to right
				jets[j].varsI[INDEX_DIR] = JET_DIR_RIGHT;

				// set velocity
				jets[j].xv = RAND_RANGE(JET_SPEED_MIN, JET_SPEED_MAX);
				jets[j].yv = 0;

			}

			j++;
		}
	}
}

//////////////////////////////////////////////////////////

void Move_Jets(void)//DONE
{
	// this function moves all active jets

	for (int i = 0; i < JET_MAX; i++)
	{
		if (jets[i].state == JET_STATE_ALIVE)
		{

			jets[i].varsI[INDEX_WORLD_X] += jets[i].xv;
			jets[i].x += jets[i].xv;

			// test for collision with player
			if (Collision_Test(jets[i].varsI[INDEX_WORLD_X] - (jets[i].width >> 1),
							   jets[i].varsI[INDEX_WORLD_Y] - (jets[i].height >> 1),
							   jets[i].width, jets[i].height,
							   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
							   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
							   flyer.width, flyer.height))
			{
				jets[i].state = JET_STATE_DEAD;
				flyer.state = FLYER_STATE_DEAD;
				Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosions(jets[i].varsI[INDEX_WORLD_X], jets[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosion_Sound();
				Start_Explosion_Sound();

				curr_game_state = FLYER_EXPLODE;
				break;
			}

			// test for collisions with flyer missles
			for (int missle = 0; missle < MISSLE_MAX; missle++)
			{
				if (missles[missle].state == MISSLE_STATE_ON && 
					missles[missle].anim_state == MISSLE_ANIM_FLYER)
				{
					// test for collision 
					if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
								       missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
									   missles[missle].width, missles[missle].height,
									   jets[i].varsI[INDEX_WORLD_X] - (jets[i].width >> 1),
									   jets[i].varsI[INDEX_WORLD_Y] - (jets[i].height >> 1),
									   jets[i].width, jets[i].height))
					{
						// kill missle
						missles[missle].state     = MISSLE_STATE_OFF;
						missles[missle].counter_1 = 0;

						// kill boat
						jets[i].state = JET_STATE_DEAD;
						Start_Explosions(jets[i].varsI[INDEX_WORLD_X], jets[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
						Start_Explosion_Sound();

						flyer_score += JET_POINTS;
						break;
					}
				}
			}

			DDraw_Lock_Back_Surface();

			// test for collision with river bank
			if (Color_Scan(jets[i].x, jets[i].y,
						   jets[i].x + jets[i].width, jets[i].y + jets[i].height,
						   COLOR_RIVER_BANK, COLOR_RIVER_BANK_2, back_buffer, back_lpitch))
			{
				// back the boat up along its last trajectory
				jets[i].varsI[INDEX_WORLD_X] -= 5 * jets[i].xv;
				jets[i].xv *= -1;

				// change direction
				if (jets[i].varsI[INDEX_DIR] == JET_DIR_LEFT)
				{
					jets[i].varsI[INDEX_DIR] = JET_DIR_RIGHT;
				}
				else
				{
					jets[i].varsI[INDEX_DIR] = JET_DIR_LEFT;
				}
			}

			DDraw_Unlock_Back_Surface();

			// fire weapon if equipped
			if (jets[i].varsI[INDEX_WEAPON] == JET_WEAPON_ON)
			{
				if (++jets[i].counter_1 > 30)
				{
					int speed;

					if (jets[i].varsI[INDEX_DIR] == JET_DIR_LEFT)
					{
						speed = jets[i].xv - 5;
					}
					else
					{
						speed = jets[i].xv + 5;
					}

					Fire_Missles(jets[i].varsI[INDEX_WORLD_X], jets[i].varsI[INDEX_WORLD_Y],
								 speed, 0, MISSLE_ANIM_ENEMY);
					Start_Missle_Enemy_Sound(jets[i].x, jets[i].y, jets[i].width, jets[i].height);
					jets[i].counter_1 = 0;
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Jets(void)//DONE
{
	// this function draws all active jets

	for (int i = 0; i < JET_MAX; i++)
	{
		if (jets[i].state == JET_STATE_ALIVE)
		{
			// transform to screen coords
			jets[i].x = jets[i].varsI[INDEX_WORLD_X] - (jets[i].width >> 1) - world_x;
			jets[i].y = jets[i].varsI[INDEX_WORLD_Y] - (jets[i].height >> 1) - world_y;

			// set correct frame
			jets[i].curr_frame = jets[i].varsI[INDEX_DIR];

			// draw patrol boat
			Draw_BOB(&jets[i], lpddsback);
		}
	}
}

///////////////////////////////////////////////////////////

void Reset_Jets(void)//DONE
{
	// this function resets all jets

	for (int i = 0; i < JET_MAX; i++)
    {
		jets[i].state = JET_STATE_DEAD;
    }
}

////////////////////////////////////////////////////////////

void Delete_Jets(void)//DONE
{
	// this function deletes all jets data

	for (int i = 0; i < JET_MAX; i++)
	{
		Destroy_BOB(&jets[i]);
	}
}

///////////////////////////////////////////////////////////

void Init_Screens(void)//done
{
	// this function loads all the bitmap screens

	// load the main screen bitmap
	Load_Bitmap_File(&bitmap8bit, "menuart/main_screen.bmp");
	Create_Bitmap(&main_screen, 0, 0, 640, 480, 8);
	Load_Image_Bitmap(&main_screen, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
	Copy_Palette(menu_pal, bitmap8bit.palette);
	Unload_Bitmap_File(&bitmap8bit);

	// load the new game screen bitmap
	Load_Bitmap_File(&bitmap8bit, "menuart/new_game_screen.bmp");
	Create_Bitmap(&new_game_screen, 0, 0, 640, 480, 8);
	Load_Image_Bitmap(&new_game_screen, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
	Unload_Bitmap_File(&bitmap8bit);

	// load the instruction screen bitmap
	Load_Bitmap_File(&bitmap8bit, "menuart/instruction_screen.bmp");
	Create_Bitmap(&instruction_screen, 0, 0, 640, 480, 8);
	Load_Image_Bitmap(&instruction_screen, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
	Unload_Bitmap_File(&bitmap8bit);

	// load the game finish screen bitmap
	Load_Bitmap_File(&bitmap8bit, "menuart/game_finish_screen.bmp");
	Create_Bitmap(&game_finish_screen, 0, 0, 640, 480, 8);
	Load_Image_Bitmap(&game_finish_screen, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
	Unload_Bitmap_File(&bitmap8bit);

	// load the game over screen bitmap
	Load_Bitmap_File(&bitmap8bit, "menuart/game_over_screen.bmp");
	Create_Bitmap(&game_over_screen, 0, 0, 640, 480, 8);
	Load_Image_Bitmap(&game_over_screen, &bitmap8bit, 0, 0, BITMAP_EXTRACT_MODE_ABS);
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Delete_Screens(void)//done
{
	// this function deletes all screen data

	Destroy_Bitmap(&main_screen);
	Destroy_Bitmap(&new_game_screen);
	Destroy_Bitmap(&instruction_screen);
	Destroy_Bitmap(&game_finish_screen);
	Destroy_Bitmap(&game_over_screen);
}

///////////////////////////////////////////////////////////

void Init_World(void)//DONE
{
	// this function initializes the world

	int i;

	// load in texture maps
	Load_Bitmap_File(&bitmap8bit, "art/world.bmp");

	// set the palette to background image palette
	Copy_Palette(world_pal, bitmap8bit.palette);

	// create the texture bob
	Create_BOB(&world, 9999, 9999, WORLD_TILE_WIDTH, WORLD_TILE_HEIGHT, NUM_TILES_WORLD,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each texture bitmap into the texture BOB object
	for (i = 0; i < NUM_TILES_WORLD; i++)
	{
		Load_Frame_BOB(&world, &bitmap8bit, i, i%4, i/4, BITMAP_EXTRACT_MODE_CELL);
	}

	// unload the texture map bitmap
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Draw_World(void)//DONE
{
	// this function draws the world

	int index_x, index_y;								// looping vars
	int start_map_x, start_map_y, end_map_x, end_map_y; // map positions 
	int offset_x, offset_y;								// pixel offsets within cell

	// compute starting map indices by dividing position by size of cell
	start_map_x = world_x >> 6;		// divide by 64
	start_map_y = world_y >> 6; 

	// compute end of map rectangle for best cast i.e. aligned on 64x64 boundary
	end_map_x = start_map_x + 10 - 1;
	end_map_y = start_map_y + 7 - 1;

	// now compute number of pixels in x,y we are within the tile, i.e
	// how much is scrolled off the edge?
	offset_x = -(world_x % 64);
	offset_y = -(world_y % 64);

	// adjust end_map_x,y for offsets
	if (offset_x)
	{
		end_map_x++;
	}

	if (offset_y)
	{
		end_map_y++;
	}

	// set starting position of first upper lh texture
	int texture_x = offset_x;
	int texture_y = offset_y;

	// draw the current window
	for (index_y = start_map_y; index_y <= end_map_y; index_y++)
	{
		for (index_x = start_map_x; index_x <= end_map_x; index_x++)
		{
			// set position to blit
			world.x = texture_x;
			world.y = texture_y;
        
			// set frame
			world.curr_frame = levels[index_y][index_x] - '0';
			
			// draw the texture
			Draw_BOB(&world, lpddsback);
			
			// update texture position
			texture_x += 64;

		} // end for map_x

		// reset x postion, update y
		texture_x = offset_x;
		texture_y += 64;
	} // end for map_y
}

////////////////////////////////////////////////////////////

void Delete_World(void)//DONE
{
	// this function deletes all world tiles

	Destroy_BOB(&world);
}

///////////////////////////////////////////////////////////

void Init_Flyer(void)//DONE
{
	// this function initialized the flyer

	int i;

	// load in texture maps
	Load_Bitmap_File(&bitmap8bit, "art/flyer.bmp");

	// create the texture bob
	Create_BOB(&flyer, 9999, 9999, FLYER_TILE_WIDTH, FLYER_TILE_HEIGHT, NUM_TILES_FLYER,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each texture bitmap into the texture BOB object
	for (i = 0; i < NUM_TILES_FLYER; i++)
	{
		Load_Frame_BOB(&flyer, &bitmap8bit, i, i, 0, BITMAP_EXTRACT_MODE_CELL);
	}

	// unload the texture map bitmap
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Flyer(void)//DONE
{
	// this function sets parameters for the flyer

	// set state
	flyer.state = FLYER_STATE_ALIVE;

	// set frame
	flyer.curr_frame = 1;

	// set weapons
	flyer.varsI[INDEX_WEAPON] = FLYER_WEAPON_ON;

	// set world position
	flyer.varsI[INDEX_WORLD_X] = world_x + flyer_init_x;
	flyer.varsI[INDEX_WORLD_Y] = world_y + flyer_init_y;

	// set position
	flyer.x = flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1) - world_x;
	flyer.y = flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1) - world_y;

	// set velocity
	flyer.xv = 0;
	flyer.yv = 0;

	// set displacement
	flyer_dx = 0;
	flyer_dy = 0;
}

///////////////////////////////////////////////////////////

void Move_Flyer(void)//DONE
{
	// this function moves the flyer

	int fueling = 0;
	int scn_mid_ht = int(screen_height/2);
	int scn_mid_wd = int(screen_width/2);

	// when flyer in middle of screen move world not flyer
	if ((flyer.y < scn_mid_ht + (flyer.height >> 1)) && (flyer.y > scn_mid_ht - (flyer.height >> 1)) && ((world_y + flyer_dy) > 0))
	{
		world_y += flyer_dy;
	}
	else
	{
		flyer.y += flyer_dy;
	}
	
	flyer.x += flyer_dx;

	flyer.varsI[INDEX_WORLD_X] += flyer_dx;
	flyer.varsI[INDEX_WORLD_Y] += flyer_dy;

	// check for level finish
	if ((flyer.varsI[INDEX_WORLD_Y] <= 48) && (bridge.state == BRIDGE_STATE_DEAD))
	{
		curr_game_level++;
		curr_game_state = NEXT_LEVEL;
		return;
	}

	// check for collisions with enemy missles
	for (int missle = 0; missle < MISSLE_MAX; missle++)
	{
		if (missles[missle].state == MISSLE_STATE_ON && 
			missles[missle].anim_state == MISSLE_ANIM_ENEMY)
		{
			// test for collision 
			if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
							   missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
							   missles[missle].width, missles[missle].height,
							   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
							   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
							   flyer.width, flyer.height))
			{
				// kill pulse
				missles[missle].state     = MISSLE_STATE_OFF;
				missles[missle].counter_1 = 0;

				flyer.state = FLYER_STATE_DEAD;
				Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
				Start_Explosion_Sound();

				curr_game_state = FLYER_EXPLODE;
				break;
			}
		}
	}

	// lock surface, so we can scan it
	DDraw_Lock_Back_Surface();

	if (Color_Scan(flyer.x, flyer.y, flyer.x + flyer.width, flyer.y + flyer.height,
                   COLOR_RIVER_BANK, COLOR_RIVER_BANK_2, back_buffer, back_lpitch))
	{
		flyer.x -= flyer_dx;
		flyer.y -= flyer_dy;
		flyer.varsI[INDEX_WORLD_X] -= flyer_dx;
		flyer.varsI[INDEX_WORLD_Y] -= flyer_dy;
		flyer.state = FLYER_STATE_DEAD;
		Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
		Start_Explosion_Sound();
		curr_game_state = FLYER_EXPLODE;
	}

	// done, so unlock
	DDraw_Unlock_Back_Surface();

	// manage fuel
	for (int i = 0; i < FUEL_MAX; i++)
	{
		if (fuel[i].state == FUEL_STATE_ALIVE)
		{
			if (Collision_Test(fuel[i].varsI[INDEX_WORLD_X] - (fuel[i].width >> 1),
							   fuel[i].varsI[INDEX_WORLD_Y] - (fuel[i].height >> 1),
							   fuel[i].width, fuel[i].height,
							   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
							   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
							   flyer.width, flyer.height))
			{
				if (flyer_fuel <= 980)
				{
					flyer_fuel += 30 + curr_game_difficulty * 10;
				}
				else
				{
					flyer_fuel = 1000;
				}

				fueling = 1;

				if (!(DSound_Status_Sound(fueling_id) & (DSBSTATUS_LOOPING | DSBSTATUS_PLAYING)))
				{
					DSound_Play(fueling_id, DSBPLAY_LOOPING);
				}

				break;
			}
		}
	}

	if (!fueling)
	{
		DSound_Stop_Sound(fueling_id);
		flyer_fuel += flyer_dy;

		if (flyer_fuel < 200)
		{
			DSound_Play(fuel_low_id, DSBPLAY_LOOPING);
		}
		else
		{
			DSound_Stop_Sound(fuel_low_id);
		}
	}

	if (flyer_fuel <= 0)
	{
		flyer.state = FLYER_STATE_DEAD;
		Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
		Start_Explosion_Sound();
		curr_game_state = FLYER_EXPLODE;
	}

	// test for collision with bridge
	if (bridge.state == BRIDGE_STATE_ALIVE)
	{
		if (Collision_Test(bridge.varsI[INDEX_WORLD_X] - (bridge.width >> 1),
						   bridge.varsI[INDEX_WORLD_Y] - (bridge.height >> 1),
						   bridge.width, bridge.height,
						   flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1),
						   flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1),
						   flyer.width, flyer.height))
		{
			bridge.state = JET_STATE_DEAD;
			flyer.state = FLYER_STATE_DEAD;
			Start_Explosions(flyer.varsI[INDEX_WORLD_X], flyer.varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
			Start_Explosions(bridge.varsI[INDEX_WORLD_X], bridge.varsI[INDEX_WORLD_Y], 64, 64, 0, 0);
			Start_Explosion_Sound();
			Start_Explosion_Sound();
			curr_game_state = FLYER_EXPLODE;
		}
	}

	// make sure player is within the world
	if (world_y < UNIVERSE_MIN_Y)
	{
		world_y = UNIVERSE_MIN_Y;
	}
	if (world_y >= UNIVERSE_MAX_Y)
	{
		world_y = UNIVERSE_MAX_Y - 1;
	}

	// check if flyer is off screen
	if (flyer.x < 0 || flyer.x > (screen_width - flyer.width))
	{
		flyer.x -= flyer_dx;
		flyer.varsI[INDEX_WORLD_X] -= flyer_dx;
	}

	if (flyer.y < 0 || flyer.y > (screen_height - flyer.height))
	{
		flyer.y -= flyer_dy;
		flyer.varsI[INDEX_WORLD_Y] -= flyer_dy;
	}
}

///////////////////////////////////////////////////////////

void Draw_Flyer(void)//DONE
{
	// this function draws the flyer

	if (flyer.state == FLYER_STATE_ALIVE)
	{
		// transform to screen coords
		flyer.x = flyer.varsI[INDEX_WORLD_X] - (flyer.width >> 1) - world_x;
		flyer.y = flyer.varsI[INDEX_WORLD_Y] - (flyer.height >> 1) - world_y;

		// draw flyer
		Draw_BOB(&flyer, lpddsback);
	}
}

////////////////////////////////////////////////////////////

void Delete_Flyer(void)//DONE
{
	// this function deletes all flyer data

	Destroy_BOB(&flyer);
}

///////////////////////////////////////////////////////////

void Init_Explosions(void)//DONE
{
	// this function initializes and loads all the explosions 

	// load the explosions imagery 
	Load_Bitmap_File(&bitmap8bit, "art/explosion.bmp");

	// create the first bob
	Create_BOB(&explosions[0], 9999, 9999, EXPLOSION_TILE_WIDTH, EXPLOSION_TILE_HEIGHT, NUM_TILES_EXPLOSION,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY);

	// load animation frames
	for (int frame=0; frame < NUM_TILES_EXPLOSION; frame++)
	{
		Load_Frame_BOB(&explosions[0], &bitmap8bit, frame, frame%6, frame/6, BITMAP_EXTRACT_MODE_CELL);  
	}

	// set animation rate
	Set_Anim_Speed_BOB(&explosions[0], 1);

	// set size of burst
	explosions[0].varsI[0] = explosions[0].width;
	explosions[0].varsI[1] = explosions[0].height;

	// set state to off
	explosions[0].state = EXPLOSION_STATE_OFF;

	for (int i = 1; i < EXPLOSION_MAX; i++)
    {
		memcpy(&explosions[i], &explosions[0], sizeof(BOB));
    }

	// unload data infile
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Explosions(int x, int y, int width, int height, int xv, int yv)//DONE
{
	// this function starts an explosion

	// scan for an explosion to initialize
	for (int i = 0; i < EXPLOSION_MAX; i++)
    {
		if (explosions[i].state == EXPLOSION_STATE_OFF)
		{
			// set animation rate
			Set_Anim_Speed_BOB(&explosions[i], 1);
			explosions[i].curr_frame = 0;
    
			// set velocity
			Set_Vel_BOB(&explosions[i], xv, yv);
    
			// set position
			Set_Pos_BOB(&explosions[i], 0,0);
			explosions[i].varsI[INDEX_WORLD_X] = x;
			explosions[i].varsI[INDEX_WORLD_Y] = y;
    
			// set size
			explosions[i].varsI[0] = width;
			explosions[i].varsI[1] = height;

			// turn on
			explosions[i].state = EXPLOSION_STATE_ON;

			return;
		}
	}
}

///////////////////////////////////////////////////////////

void Move_Explosions(void)//DONE
{
	// this function moves all the explosions

	for (int i = 0; i < EXPLOSION_MAX; i++)
    {
		if (explosions[i].state == EXPLOSION_STATE_ON)
		{
			explosions[i].varsI[INDEX_WORLD_X] += explosions[i].xv;
			explosions[i].varsI[INDEX_WORLD_Y] += explosions[i].yv;

			if (explosions[i].curr_frame >= explosions[i].num_frames - 1)
			{
				explosions[i].state = EXPLOSION_STATE_OFF;
			}
        }
    }
}

///////////////////////////////////////////////////////////

void Draw_Explosions(void)//DONE
{
	// this function draws all the explosions

	for (int i = 0; i < EXPLOSION_MAX; i++)
    {
		// test if explosions are in flight
		if (explosions[i].state == EXPLOSION_STATE_ON)
        {
			// transform explosions to screen space
			explosions[i].x = explosions[i].varsI[INDEX_WORLD_X] - (explosions[i].width >> 1) - world_x;
			explosions[i].y = explosions[i].varsI[INDEX_WORLD_Y] - (explosions[i].height >> 1) - world_y;

			// is scaling needed
			if (explosions[i].varsI[0] != explosions[i].width || 
				explosions[i].varsI[1] != explosions[i].height)
            {
				// draw the burst scaled
				Draw_Scaled_BOB(&explosions[i], explosions[i].varsI[0], explosions[i].varsI[1], lpddsback);
            }
			else // draw normal
			{
				Draw_BOB(&explosions[i], lpddsback);
			}
         
			// animate the explosion
			Animate_BOB(&explosions[i]);
        }
    }
}

///////////////////////////////////////////////////////////

void Reset_Explosions(void)//DONE
{
	// this function resets all the explosions

	for (int i = 0; i < EXPLOSION_MAX; i++)
    {
		explosions[i].state = EXPLOSION_STATE_OFF;
    }
}


////////////////////////////////////////////////////////////

void Delete_Explosions(void)//DONE
{
	// this function deletes all jets data

	for (int i = 0; i < EXPLOSION_MAX; i++)
	{
		Destroy_BOB(&explosions[i]);
	}
}

//////////////////////////////////////////////////////

void Init_Missles(void)//DONE
{
	// this function initializes and loads all missles

	// missle animations
	static int missles_anim_flyer[] = {0, 1, 2, 3, 4, 5, 6, 7},
   			   missles_anim_enemy[]  = {8, 9, 10, 11, 12, 13, 14, 15};

	// load missle bitmap
	Load_Bitmap_File(&bitmap8bit, "art/missle.bmp");

	// create the first bob
	Create_BOB(&missles[0], 9999, 9999, MISSLE_TILE_WIDTH, MISSLE_TILE_HEIGHT, NUM_TILES_MISSLE,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_ANIM, DDSCAPS_SYSTEMMEMORY);

	// load animation frames
	for (int frame = 0; frame < NUM_TILES_MISSLE; frame++)
	{
		Load_Frame_BOB(&missles[0], &bitmap8bit, frame, frame%8, frame/8, BITMAP_EXTRACT_MODE_CELL);
	}

	// set animation rate
	Set_Anim_Speed_BOB(&missles[0], 1);

	// load animations
	Load_Animation_BOB(&missles[0], MISSLE_ANIM_FLYER, 8, missles_anim_flyer);
	Load_Animation_BOB(&missles[0], MISSLE_ANIM_ENEMY, 8, missles_anim_enemy);

	// set state to off
	missles[0].state = MISSLE_STATE_OFF;

	for (int i = 1; i < MISSLE_MAX; i++)
    {
		memcpy(&missles[i], &missles[0], sizeof(BOB));
    }

	// unload data infile
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Fire_Missles(int x, int y, int xv, int yv, int source)//DONE
{
	// this function fires a missle at the given starting
	// position and velocity

	// scan for a missle that is available
	for (int i = 0; i < MISSLE_MAX; i++)
    {
		// is this one available
		// test if plasma pulse is in flight
		if (missles[i].state == MISSLE_STATE_OFF)
		{
			// start this one up, note the use of world coords
			missles[i].varsI[INDEX_WORLD_X] = x - (missles[0].width >> 1);
			missles[i].varsI[INDEX_WORLD_Y] = y - (missles[0].height >> 1);
			missles[i].x = missles[i].varsI[INDEX_WORLD_X] - (missles[i].width >> 1) - world_x;
			missles[i].y = missles[i].varsI[INDEX_WORLD_Y] - (missles[i].height >> 1) - world_y;       
			missles[i].xv = xv;
			missles[i].yv = yv;
			missles[i].curr_frame = 0;
			missles[i].state = MISSLE_STATE_ON;
			missles[i].counter_1 = 0; // used to track distance

			// set animation and typing info for collision engine
			missles[i].anim_state = source;
			Set_Animation_BOB(&missles[i], source);

			return;
       }
    }
}

///////////////////////////////////////////////////////////

void Move_Missles(void)//DONE
{
	// this function moves all missles

	for (int i = 0; i < MISSLE_MAX; i++)
    {
		// test if missles are in flight
		if (missles[i].state == MISSLE_STATE_ON)
        {
			// move the pulse 
			missles[i].varsI[INDEX_WORLD_X] += missles[i].xv;
			missles[i].varsI[INDEX_WORLD_Y] += missles[i].yv;
			missles[i].x += missles[i].xv;
			missles[i].y += missles[i].yv;

			// test for boundaries
			if ((++missles[i].counter_1 > MISSLE_RANGE_FLYER) || 
				(missles[i].varsI[INDEX_WORLD_X] > UNIVERSE_MAX_X) || 
				(missles[i].varsI[INDEX_WORLD_X] < UNIVERSE_MIN_X) ||
				(missles[i].varsI[INDEX_WORLD_Y] > (UNIVERSE_MAX_Y + 512)) || 
				(missles[i].varsI[INDEX_WORLD_Y] < UNIVERSE_MIN_Y))
            {
				// kill the pulse
				missles[i].state = MISSLE_STATE_OFF;
				missles[i].counter_1 = 0;
				continue;
			}

			// lock surface, so we can scan it
			DDraw_Lock_Back_Surface();

			if (Color_Scan(missles[i].x, missles[i].y,
						   missles[i].x + missles[i].width, missles[i].y + missles[i].height,
						   COLOR_RIVER_BANK, COLOR_RIVER_BANK_2, back_buffer, back_lpitch))
			{
				missles[i].varsI[INDEX_WORLD_X] -= missles[i].xv;
				missles[i].varsI[INDEX_WORLD_Y] -= missles[i].yv;
				missles[i].x -= missles[i].xv;
				missles[i].y -= missles[i].yv;
				missles[i].state = MISSLE_STATE_OFF;
				missles[i].counter_1 = 0;
			}

			// done, so unlock
			DDraw_Unlock_Back_Surface();
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Missles(void)//DONE
{
	// this function draws all missles

	for (int i = 0; i < MISSLE_MAX; i++)
    {
		// test if missles are in flight
		if (missles[i].state == MISSLE_STATE_ON)
        {
			// transform to screen coords
			missles[i].x = missles[i].varsI[INDEX_WORLD_X] - (missles[i].width >> 1) - world_x;
			missles[i].y = missles[i].varsI[INDEX_WORLD_Y] - (missles[i].height >> 1) - world_y;

			// draw the pulse
			Draw_BOB(&missles[i], lpddsback);
         
	        // animate the pulse
		    Animate_BOB(&missles[i]);
        }
    }
}

/////////////////////////////////////////////////////////

void Reset_Missles(void)//DONE
{
	// this function resets all missles

	for (int i = 0; i < MISSLE_MAX; i++)
    {
		missles[i].state = MISSLE_STATE_OFF;
		missles[i].counter_1 = 0;
    }
}

///////////////////////////////////////////////////////////

void Delete_Missles(void)//DONE
{
	// this function deletes all missles data

	for (int i = 0; i < MISSLE_MAX; i++)
	{
		Destroy_BOB(&missles[i]);
	}
}

///////////////////////////////////////////////////////////

void Init_Fuel(void)//DONE
{
	// this function initializes all fuel

	int i;

	// load in fuel tiles
	Load_Bitmap_File(&bitmap8bit, "art/fuel.bmp");

	// create the fuel bob
	Create_BOB(&fuel[0], 9999, 9999, FUEL_TILE_WIDTH, FUEL_TILE_HEIGHT, NUM_TILES_FUEL,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each fuel bitmap into the fuel BOB object
	Load_Frame_BOB(&fuel[0], &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

	// set state to off
	fuel[0].state = FUEL_STATE_DEAD;

	// make copies
	for (i = 1; i < FUEL_MAX; i++)
	{
		memcpy(&fuel[i], &fuel[0], sizeof(BOB));
	}

	// unload the fuel tiles
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Fuel(void)//DONE
{
	// this function sets parameters for active fuel

	int j = 0;
	int num_enemy = num_enemies[curr_game_level];

	for (int i = 0; i < num_enemy; i++)
	{
		if (enemies[curr_game_level][i][ENEMY_TYPE] == TYPE_FUEL)
		{
			// set state
			fuel[j].state = FUEL_STATE_ALIVE;

			// set position
			fuel[j].varsI[INDEX_WORLD_X] = enemies[curr_game_level][i][POS_X];
			fuel[j].varsI[INDEX_WORLD_Y] = enemies[curr_game_level][i][POS_Y];
			fuel[j].x = fuel[j].varsI[INDEX_WORLD_X] - (fuel[j].width >> 1) - world_x;
			fuel[j].y = fuel[j].varsI[INDEX_WORLD_Y] - (fuel[j].height >> 1) - world_y;
			fuel[j].xv = 0;
			fuel[j].yv = 0;

			j++;
		}
	}
}

////////////////////////////////////////////////////////////

void Move_Fuel(void)//DONE
{
	// this function moves all fuel

	for (int i = 0; i < FUEL_MAX; i++)
    {
		// test if fuel is alive
		if (fuel[i].state == FUEL_STATE_ALIVE)
        {
			// test for collisions with flyer missles
			for (int missle = 0; missle < MISSLE_MAX; missle++)
			{
				if (missles[missle].state == MISSLE_STATE_ON && 
					missles[missle].anim_state == MISSLE_ANIM_FLYER)
				{
					// test for collision 
					if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
								       missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
									   missles[missle].width, missles[missle].height,
									   fuel[i].varsI[INDEX_WORLD_X] - (fuel[i].width >> 1),
									   fuel[i].varsI[INDEX_WORLD_Y] - (fuel[i].height >> 1),
									   fuel[i].width, fuel[i].height))
					{
						// kill missle
						missles[missle].state     = MISSLE_STATE_OFF;
						missles[missle].counter_1 = 0;

						// kill fuel
						fuel[i].state = FUEL_STATE_DEAD;

						Start_Explosions(fuel[i].varsI[INDEX_WORLD_X], fuel[i].varsI[INDEX_WORLD_Y], EXPLOSION_WIDTH, EXPLOSION_HEIGHT, 0, 0);
						Start_Explosion_Sound();

						flyer_score += FUEL_POINTS;
						break;
					}
				}
			}
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Fuel(void)//DONE
{
	// this function draws all fuel

	for (int i = 0; i < FUEL_MAX; i++)
    {
		// test if fuel is aluive
		if (fuel[i].state == FUEL_STATE_ALIVE)
        {
			// transform to screen coords
			fuel[i].x = fuel[i].varsI[INDEX_WORLD_X] - (fuel[i].width >> 1) - world_x;
			fuel[i].y = fuel[i].varsI[INDEX_WORLD_Y] - (fuel[i].height >> 1) - world_y;
	
			// draw the Fuel
			Draw_BOB(&fuel[i], lpddsback);
        }
    }
}

////////////////////////////////////////////////////////////

void Reset_Fuel(void)//DONE
{
	// this function resets all fuel

	for (int i = 0; i < FUEL_MAX; i++)
    {
		fuel[i].state = FUEL_STATE_DEAD;
	}
}

////////////////////////////////////////////////////////////

void Delete_Fuel(void)//DONE
{
	// this function deletes all fuel data

	for (int i = 0; i < FUEL_MAX; i++)
	{
		Destroy_BOB(&fuel[i]);
	}
}

///////////////////////////////////////////////////////////

void Init_Bridge(void)//DONE
{
	// this function initializes bridge

	int i;

	// load in bridge tiles
	Load_Bitmap_File(&bitmap8bit, "art/bridge.bmp");

	// create the bridge bob
	Create_BOB(&bridge, 9999, 9999, BRIDGE_TILE_WIDTH, BRIDGE_TILE_HEIGHT, NUM_TILES_BRIDGE,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, COLOR_TRANSPARENT);

	// load each bridge bitmap into the bridge BOB object
	Load_Frame_BOB(&bridge, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

	// set state to off
	bridge.state = BRIDGE_STATE_DEAD;

	// unload the bridge tiles
	Unload_Bitmap_File(&bitmap8bit);
}

///////////////////////////////////////////////////////////

void Start_Bridge(void)//DONE
{
	// this function sets parameters for bridge

	// set state
	bridge.state = BRIDGE_STATE_ALIVE;

	// set position
	bridge.varsI[INDEX_WORLD_X] = bridge_init_world_x;
	bridge.varsI[INDEX_WORLD_Y] = bridge_init_world_y;
	bridge.x = bridge.varsI[INDEX_WORLD_X] - (bridge.width >> 1) - world_x;
	bridge.y = bridge.varsI[INDEX_WORLD_Y] - (bridge.height >> 1) - world_y;
	bridge.xv = 0;
	bridge.yv = 0;
}

////////////////////////////////////////////////////////////

void Move_Bridge(void)//DONE
{
	// this function moves bridge

	// test for collisions with flyer missles
	for (int missle = 0; missle < MISSLE_MAX; missle++)
	{
		if (missles[missle].state == MISSLE_STATE_ON && 
			missles[missle].anim_state == MISSLE_ANIM_FLYER)
		{
			// test for collision 
			if (Collision_Test(missles[missle].varsI[INDEX_WORLD_X] - (missles[missle].width >> 1), 
							   missles[missle].varsI[INDEX_WORLD_Y] - (missles[missle].height >> 1), 
							   missles[missle].width, missles[missle].height,
							   bridge.varsI[INDEX_WORLD_X] - (bridge.width >> 1),
							   bridge.varsI[INDEX_WORLD_Y] - (bridge.height >> 1),
							   bridge.width, bridge.height))
			{
				// kill missle
				missles[missle].state     = MISSLE_STATE_OFF;
				missles[missle].counter_1 = 0;

				// kill bridge
				bridge.state = BRIDGE_STATE_DEAD;

				Start_Explosions(bridge.varsI[INDEX_WORLD_X], bridge.varsI[INDEX_WORLD_Y], 64, 64, 0, 0);
				Start_Explosion_Sound();

				flyer_score += BRIDGE_POINTS;
				break;
			}
		}
	}
}

///////////////////////////////////////////////////////////

void Draw_Bridge(void)//DONE
{
	// this function draws bridge

	// test if bridge is aluive
	if (bridge.state == BRIDGE_STATE_ALIVE)
    {
		// transform to screen coords
		bridge.x = bridge.varsI[INDEX_WORLD_X] - (bridge.width >> 1) - world_x;
		bridge.y = bridge.varsI[INDEX_WORLD_Y] - (bridge.height >> 1) - world_y;

		// draw the Bridge
		Draw_BOB(&bridge, lpddsback);
    }
}

////////////////////////////////////////////////////////////

void Reset_Bridge(void)//DONE
{
	// this function resets bridge

	bridge.state = BRIDGE_STATE_DEAD;
}

////////////////////////////////////////////////////////////

void Delete_Bridge(void)//DONE
{
	// this function deletes all bridge data

	Destroy_BOB(&bridge);
}

///////////////////////////////////////////////////////////

void Init_Info(void)//DONE
{
	// this function initializes the info panel

	// load in texture maps
	Load_Bitmap_File(&bitmap8bit, "art/info.bmp");

	// create the boat bob
	Create_BOB(&info, 9999, 9999, 640, 32, 1,
			   BOB_ATTR_VISIBLE | BOB_ATTR_MULTI_FRAME, DDSCAPS_SYSTEMMEMORY, 1);

	// load each boat bitmap into the boat BOB object
	Load_Frame_BOB(&info, &bitmap8bit, 0, 0, 0, BITMAP_EXTRACT_MODE_CELL);

	// unload the texture map bitmap
	Unload_Bitmap_File(&bitmap8bit);

	info.x = 0;
	info.y = 448;
}

///////////////////////////////////////////////////////////

void Draw_Info(void)//DONE
{
	// this function draws all the info panel

	char score[16];					// hold score
	static int red_glow = 0;		// low fuel warning
	static int warning_count = 0;

	// draw panel background
	Draw_BOB(&info, lpddsback);

	// build up scrore string
	sprintf(score, "0000000%d", flyer_score);

	// build up final string
	sprintf(buffer, "SCORE  %s", &score[strlen(score)-8]);
	Draw_Text_GDI(buffer, 15, 460, RGB(255, 255, 255), lpddsback);

	// draw fuel
	sprintf(buffer, "FUEL  %d%%", flyer_fuel/10);

	if (flyer_fuel > 200)
	{
		Draw_Text_GDI(buffer, 350-8*strlen(buffer), 460, RGB(255, 255, 255), lpddsback);
	}
	else
	{
		Draw_Text_GDI(buffer, 350-8*strlen(buffer), 460, RGB(255, red_glow, red_glow), lpddsback);
	}

	// update red glow
	if ((red_glow+=15) > 255)
	{
		red_glow = 0;
	}

	// draw ships info
	sprintf(buffer, "SHIPS  %d", flyer_ships);
	Draw_Text_GDI(buffer, 470, 460, RGB(255, 255, 255), lpddsback);

	// draw level info
	sprintf(buffer, "LEVEL  %d", (curr_game_level + 1));
	Draw_Text_GDI(buffer, 550, 460, RGB(255, 255, 255), lpddsback);
}

///////////////////////////////////////////////////////////

void Delete_Info(void)//DONE
{
	// this function deletes all info panel data

	Destroy_BOB(&info);
}

///////////////////////////////////////////////////////////

int Init_Sounds(void)
{
	// this function loads all the sounds

	int i;

	// load engine sound
	if ((engine_id = DSound_Load_WAV("sounds/engine.wav")) == -1)
	{
		return(0);
	}

	DSound_Set_Volume(engine_id, 60);
    
	// load fueling sound
	if ((fueling_id = DSound_Load_WAV("sounds/fueling.wav")) == -1)
	{
		return(0);
	}

	// load fuel low sound
	if ((fuel_low_id = DSound_Load_WAV("sounds/fuel_low.wav")) == -1)
	{
		return(0);
	}

	DSound_Set_Volume(fuel_low_id, 20);
	DSound_Set_Freq(fuel_low_id, 6000);

	// load enemy missle sound
	if ((missle_enemy_ids[0] = DSound_Load_WAV("sounds/missle.wav")) == -1)
	{
		return(0);
	}

	DSound_Set_Volume(missle_enemy_ids[0], 20);

	for (i = 1; i < MAX_MISSLE_SOUNDS; i++)
    {
		missle_enemy_ids[i] = DSound_Replicate_Sound(missle_enemy_ids[0]);
		DSound_Set_Volume(missle_enemy_ids[i], 20);
    }

	// load flyer missle sound
	if ((missle_flyer_ids[0] = DSound_Load_WAV("sounds/missle.wav")) == -1)
	{
		return(0);
	}

	DSound_Set_Volume(missle_flyer_ids[0], 40);

	for (i = 1; i < MAX_MISSLE_SOUNDS; i++)
    {
		missle_flyer_ids[i] = DSound_Replicate_Sound(missle_flyer_ids[0]);
		DSound_Set_Volume(missle_flyer_ids[i], 40);
    }

	// load explosion sound
	if ((explosion_ids[0] = DSound_Load_WAV("sounds/explosion.wav")) == -1)
	{
		return(0);
	}

	DSound_Set_Volume(explosion_ids[0], 70);

	for (i = 1; i < MAX_EXPLOSION_SOUNDS; i++)
    {
		explosion_ids[i] = DSound_Replicate_Sound(explosion_ids[0]);
		DSound_Set_Volume(explosion_ids[0], 70);
    }

	return(1);
}

///////////////////////////////////////////////////////////

void Start_Missle_Enemy_Sound(int x, int y, int width, int height)
{
	// check if initiating onject is off screen
	if (x < -width || y < -height || x > screen_width || y > screen_height)
	{
		return;
	}

	// find empty slot and start sound
	for (int i = 0; i < MAX_MISSLE_SOUNDS; i++)
	{
		if (DSound_Status_Sound(missle_enemy_ids[i]) == 0)
		{
			DSound_Play(missle_enemy_ids[i]);
			break;
		}
	}
}

///////////////////////////////////////////////////////////

void Start_Missle_Flyer_Sound(int x, int y, int width, int height)
{
	// find empty slot and start sound
	for (int i = 0; i < MAX_MISSLE_SOUNDS; i++)
	{
		if (DSound_Status_Sound(missle_flyer_ids[i]) == 0)
		{
			DSound_Play(missle_flyer_ids[i]);
			break;
		}
	}
}

///////////////////////////////////////////////////////////

void Start_Explosion_Sound(void)
{
	// find empty slot and start sound
	for (int i = 0; i < MAX_EXPLOSION_SOUNDS; i++)
	{
		if (DSound_Status_Sound(explosion_ids[i]) == 0)
		{
			DSound_Play(explosion_ids[i]);
			break;
		}
	}
}